VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "cMP3"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Attribute VB_Ext_KEY = "SavedWithClassBuilder6" ,"Yes"
Attribute VB_Ext_KEY = "Top_Level" ,"Yes"
'##   PIRATEADO Y MODIFICADO POR RAUL MARTINEZ ##

' *************************************************************************************************
' This MP3 tagging module reads, writes and deletes ID3 v 1.0, 1.1, 2.2, 2.3 and 2.4 tags,
' with support for extended tag headers, unsynchronized tags, tag footers, appended tags and
' MPEG file information.
'
' This module is a part of Magic MP3 Tagger. Visit www.magic-tagger.com if you're interested in
' this top MP3 tagger!
'
' Copyright by Mathias Kunter.
' You're free to use this module within your programs. These programs may also be commercial.
' The only condition for usage of this module is that you show the following line within the
' credits of your program:
'
' ID3 tagging module by Mathias Kunter (www.magic-tagger.com)
'
' *************************************************************************************************
Option Explicit
Private HasLyrics3Tag  As Boolean
Private PosTagLyr As String
Private m_sFileName As String


Private Type tOtherInfo
    Lyrics As String
    Quality As String
    Situation As String
    Mood As String
    Author As String
    OtherInfo As String
    Images As String
    UnknownFields As String
    Album As String
    Artist As String
    Title As String
End Type

Private m_LastUpdate As String
Private m_fReadMPEGInfo As Boolean


'***** The ID3 tag for data input / output with this module *****
Private Type ID3Tag
    Artist As String
    Album As String
    Title As String
    SongYear As String
    Genre As String
    Comment As String
    TrackNr As String

    'The following items are only used with ID3 v2 tags.
    EncodedBy As String
    LinkTo As String
    Copyright As String
    Composer As String
    OrigArtist As String
End Type

Private Type MPEGInfo
    MPEGVersion As String
    HasCRC As Boolean
    bitrate As Long     'kbit / sec (where k = 1000, not 1024)
    Frequency As Long   'hz
    HasVBR As Boolean
    ChannelMode As String
    IsCopyrighted As Boolean
    IsOriginal As Boolean
    HasEmphasis As Boolean
    length As Long      'sec
    ID3v1Version As Long    '-1 = no tag
    ID3v2Version As Long    '-1 = no tag
    FileSize As Long    'bytes
    LengthTime As String
    SizeMB As String
End Type

Private Enum ID3v2_Version
    VERSION_2_3
    VERSION_2_4
End Enum

Private myTag As ID3Tag
Private myOtherinfo As tOtherInfo
Private myMPEGInfo As MPEGInfo

Private Const ID3_GENRE_COUNT As Long = 148



'***** ID3v1 Types *****
Private Type v1Tag
    Identifier(2) As Byte
    Title(29) As Byte
    Artist(29) As Byte
    Album(29) As Byte
    SongYear(3) As Byte
    Comment(29) As Byte
    Genre As Byte
End Type



'***** ID3v2 Types *****

'Tag header for all versions (2.2 - 2.4)
'The v 2.4 footer also uses this structure, since it's a copy of the header.
Private Type v2TagHeader
    Identifier(2) As Byte
    Version(1) As Byte
    Flags As Byte
    Size(3) As Byte
End Type

'Extended tag header for v 2.3
Private Type v2_3ExtHeader
    Size(3) As Byte
    ExtendedFlags(1) As Byte
    PaddingSize(3) As Byte
End Type

'Extended tag header for v 2.4
Private Type v2_4ExtHeader
    Size(3) As Byte
    NumFlags As Byte
    ExtendedFlags As Byte
End Type

'Frame header for v 2.2
Private Type v2_2FrameHeader
    Ident(2) As Byte
    Size(2) As Byte
End Type

'Frame header for v 2.3 and v 2.4
Private Type v2_34FrameHeader
    Ident(3) As Byte
    Size(3) As Byte
    Flags(1) As Byte
End Type

'A v 2.x frame in memory.
Private Type v2MemFrame
    id As String
    Size As Long
    data() As Byte
End Type

Private Type v2TagHeaderFlags
    bUnsynch As Boolean
    bExtHeader As Boolean
    bFooter As Boolean
    bUpdateTag As Boolean
End Type

Private Type v2TagSizes
    ReadSize As Long
    ExtHeaderSize As Long
    FooterSize As Long
End Type

Private Enum v2_StrEncoding
    ENC_ISO = 0
    ENC_UNICODE_UTF16_BOM = 1
    ENC_UNICODE_UTF16 = 2
    ENC_UNICODE_UTF8 = 3
End Enum

'The frames stored in memory, used when updating a currently existing v2 tag.
Private mMemFrames() As v2MemFrame, mMemFraCnt As Long

Private Const cGenreString As String = _
"Blues#Classic Rock#Country#Dance#Disco#Funk#Grunge#Hip-Hop#Jazz#Metal#New Age#Oldies#Other#Pop#" & _
"R&B#Rap#Reggae#Rock#Techno#Industrial#Alternative#Ska#Death Metal#Pranks#Soundtrack#Euro-Techno#" & _
"Ambient#Trip-Hop#Vocal#Jazz+Funk#Fusion#Trance#Classical#Instrumental#Acid#House#Game#Sound Clip#" & _
"Gospel#Noise#Alt. Rock#Bass#Soul#Punk#Space#Meditative#Instrumental Pop#Instrumental Rock#Ethnic#" & _
"Gothic#Darkwave#Techno-Industrial#Electronic#Pop-Folk#Eurodance#Dream#Southern Rock#Comedy#Cult#" & _
"Gangsta#Top 40#Christian Rap#Pop/Funk#Jungle#Native American#Cabaret#New Wave#Psychadelic#Rave#" & _
"Showtunes#Trailer#Lo-Fi#Tribal#Acid Punk#Acid Jazz#Polka#Retro#Musical#Rock & Roll#Hard Rock#Folk#" & _
"Folk-Rock#National Folk#Swing#Fast Fusion#Bebob#Latin#Revival#Celtic#Bluegrass#Avantgarde#Gothic Rock#" & _
"Progressive Rock#Psychedelic Rock#Symphonic Rock#Slow Rock#Big Band#Chorus#Easy Listening#Acoustic#" & _
"Humour#Speech#Chanson#Opera#Chamber Music#Sonata#Symphony#Booty Bass#Primus#Porn Groove#Satire#Slow Jam#" & _
"Club#Tango#Samba#Folklore#Ballad#Power Ballad#Rhythmic Soul#Freestyle#Duet#Punk Rock#Drum Solo#A capella#" & _
"Euro-House#Dance Hall#Goa#Drum & Bass#Club-House#Hardcore#Terror#Indie#BritPop#Negerpunk#Polsk Punk#" & _
"Beat#Christian Gangsta#Heavy Metal#Black Metal#Crossover#Contemporary C#Christian Rock#Merengue#Salsa#" & _
"Thrash Metal#Anime#JPop#SynthPop"
Private GenreField() As String, bBuiltGenre As Boolean


'API
Private Const GENERIC_READ = &H80000000
Private Const GENERIC_WRITE = &H40000000
Private Const FILE_SHARE_READ = &H1
Private Const OPEN_EXISTING = &H3

Private Const FILE_BEGIN = 0
Private Const FILE_CURRENT = 1
Private Const FILE_END = 2

Private Const FILE_ATTRIBUTE_READONLY = &H1
Private Const INVALID_HANDLE_VALUE = -1

Private Declare Function CreateFile Lib "kernel32" Alias "CreateFileA" (ByVal lpFileName As String, ByVal dwDesiredAccess As Long, ByVal dwShareMode As Long, lpSecurityAttributes As Any, ByVal dwCreationDisposition As Long, ByVal dwFlagsAndAttributes As Long, ByVal hTemplateFile As Long) As Long
Private Declare Function GetFileSize Lib "kernel32" (ByVal hFile As Long, lpFileSizeHigh As Long) As Long
Private Declare Function SetFilePointer Lib "kernel32" (ByVal hFile As Long, ByVal lDistanceToMove As Long, lpDistanceToMoveHigh As Long, ByVal dwMoveMethod As Long) As Long
Private Declare Function ReadFile Lib "kernel32" (ByVal hFile As Long, lpBuffer As Any, ByVal nNumberOfBytesToRead As Long, lpNumberOfBytesRead As Long, lpOverlapped As Any) As Long
Private Declare Function WriteFile Lib "kernel32" (ByVal hFile As Long, lpBuffer As Any, ByVal nNumberOfBytesToWrite As Long, lpNumberOfBytesWritten As Long, lpOverlapped As Any) As Long
Private Declare Function CloseHandle Lib "kernel32" (ByVal hObject As Long) As Long
Private Declare Function SetEndOfFile Lib "kernel32" (ByVal hFile As Long) As Long
Private Declare Function GetFileAttributes Lib "kernel32" Alias "GetFileAttributesA" (ByVal lpFileName As String) As Long
Private Declare Function SetFileAttributes Lib "kernel32" Alias "SetFileAttributesA" (ByVal lpFileName As String, ByVal dwFileAttributes As Long) As Long

Private Declare Sub CopyMemory Lib "kernel32" Alias "RtlMoveMemory" (Destination As Any, Source As Any, ByVal length As Long)
Private Type SECURITY_ATTRIBUTES
        nLength As Long
        lpSecurityDescriptor As Long
        bInheritHandle As Long
End Type
Private Const FILE_FLAG_BACKUP_SEMANTICS = &H2000000


Public Property Let Read_MPEGInfo(Tag As Boolean)
   m_fReadMPEGInfo = Tag
End Property
Public Property Get Read_MPEGInfo() As Boolean
   Read_MPEGInfo = m_fReadMPEGInfo
End Property

Public Property Get MPEG_Version() As String
   MPEG_Version = myMPEGInfo.MPEGVersion
End Property


Public Property Get MPEG_FileSizeMB() As String
   MPEG_FileSizeMB = myMPEGInfo.SizeMB
End Property

Public Property Get MPEG_Bit_Rate() As Long
   MPEG_Bit_Rate = myMPEGInfo.bitrate
End Property

Public Property Get MPEG_Frequency() As Long
   MPEG_Frequency = myMPEGInfo.Frequency
End Property

Public Property Get MPEG_Emphasis() As String
   MPEG_Emphasis = myMPEGInfo.HasEmphasis
End Property
Public Property Get MPEG_Original() As String
   MPEG_Original = myMPEGInfo.IsOriginal
End Property
Public Property Get MPEG_Copyrighted() As String
   MPEG_Copyrighted = myMPEGInfo.IsCopyrighted
End Property
Public Property Get MPEG_CRCs() As String
   MPEG_CRCs = myMPEGInfo.HasCRC
End Property
Public Property Get MPEG_DurationTime() As String
   MPEG_DurationTime = myMPEGInfo.LengthTime
End Property
Public Property Get MPEG_VBR() As Boolean
   MPEG_VBR = myMPEGInfo.HasVBR
End Property
Public Property Get MPEG_ChannelMode() As String
   MPEG_ChannelMode = myMPEGInfo.ChannelMode
End Property


Public Property Get TagID3V1() As String
   If myMPEGInfo.ID3v1Version = -1 Then
     TagID3V1 = "none"
   Else
     TagID3V1 = "version 1." & myMPEGInfo.ID3v1Version
   End If
End Property

Public Property Get TagID3V2() As String
   If myMPEGInfo.ID3v2Version = -1 Then
     TagID3V2 = "none"
   Else
     TagID3V2 = "version 2." & myMPEGInfo.ID3v2Version
   End If
End Property


Public Property Get Has_Lyrics3_Tag() As Boolean
   Has_Lyrics3_Tag = HasLyrics3Tag
End Property

Public Property Get Lyrics() As String
   Lyrics = myOtherinfo.Lyrics
End Property

Public Property Let Lyrics(NewLyrics As String)
   myOtherinfo.Lyrics = NewLyrics
End Property

Public Property Get Quality() As String
   Quality = myOtherinfo.Quality
End Property

Public Property Let Quality(NewQuality As String)
   myOtherinfo.Quality = NewQuality
End Property

Public Property Get Situation() As String
   Situation = myOtherinfo.Situation
End Property

Public Property Let Situation(NewSituation As String)
   myOtherinfo.Situation = NewSituation
End Property

Public Property Get Mood() As String
   Mood = myOtherinfo.Mood
End Property

Public Property Let Mood(NewMood As String)
   myOtherinfo.Mood = NewMood
End Property

Public Property Get Author() As String
   Author = myOtherinfo.Author
End Property

Public Property Let Author(NewAuthor As String)
   myOtherinfo.Author = NewAuthor
End Property

Public Property Get OtherInfo() As String
   OtherInfo = myOtherinfo.OtherInfo
End Property

Public Property Let OtherInfo(NewOtherInfo As String)
   myOtherinfo.OtherInfo = NewOtherInfo
End Property

Public Property Get Images() As String
   Images = myOtherinfo.Images
End Property

Public Property Let Images(NewImages As String)
   myOtherinfo.Images = NewImages
End Property

Public Property Let Year(ByVal NewYear As String)
    myTag.SongYear = NewYear
End Property

Public Property Get Year() As String
   Year = myTag.SongYear
End Property

Public Property Get DurationInSecs()
   DurationInSecs = myMPEGInfo.length
End Property

Public Property Let Genre(ByVal NewGenre As String)
    myTag.Genre = NewGenre
End Property

Public Property Get Genre() As String
  Genre = myTag.Genre
End Property


Public Property Let Comment(ByVal NewComment As String)
    myTag.Comment = NewComment
End Property

Public Property Get Comment() As String
    Comment = myTag.Comment
End Property

Public Property Let Album(ByVal NewAlbum As String)
    myTag.Album = NewAlbum
End Property

Public Property Get Album() As String
    Album = myTag.Album
End Property

Public Property Let TrackNr(ByVal NewTrack As String)
    myTag.TrackNr = NewTrack
End Property

Public Property Get TrackNr() As String
    TrackNr = myTag.TrackNr
End Property

Public Property Let Composer(ByVal NewComposer As String)
    myTag.Composer = NewComposer
End Property

Public Property Get Composer() As String
    Composer = myTag.Composer
End Property

Public Property Let OrigArtist(ByVal NewOrigArtist As String)
    myTag.OrigArtist = NewOrigArtist
End Property

Public Property Get OrigArtist() As String
    OrigArtist = myTag.OrigArtist
End Property

Public Property Let Copyright(ByVal NewCopyright As String)
    myTag.Copyright = NewCopyright
End Property

Public Property Get Copyright() As String
    Copyright = myTag.Copyright
End Property

Public Property Let LinkTo(ByVal NewlinkTo As String)
    myTag.LinkTo = NewlinkTo
End Property

Public Property Get LinkTo() As String
    LinkTo = myTag.LinkTo
End Property

Public Property Let EncodedBy(ByVal NewEncodedBy As String)
    myTag.EncodedBy = NewEncodedBy
End Property

Public Property Get EncodedBy() As String
    EncodedBy = myTag.EncodedBy
End Property

Public Property Let Artist(ByVal NewArtist As String)
    myTag.Artist = NewArtist
End Property

Public Property Get Artist() As String
    Artist = myTag.Artist
End Property

Public Property Let Title(ByVal NewTitle As String)
    myTag.Title = NewTitle
End Property

Public Property Get Title() As String
    Title = myTag.Title
End Property

Public Property Get filename() As String
    filename = m_sFileName
End Property

Public Property Get FileSize() As String
    FileSize = myMPEGInfo.FileSize
End Property


'**********************************ID3v 1.0, 1.1****************************************

Private Function ReadID3v1(ByVal filename As String, ByRef outTag As ID3Tag) As Boolean
    Dim fh As Long, fsize As Long
    Dim RdTag As v1Tag

    fh = CreateFile(filename, GENERIC_READ, FILE_SHARE_READ, ByVal 0, OPEN_EXISTING, 0, 0)
    If fh = INVALID_HANDLE_VALUE Then Exit Function
    fsize = GetFileSize(fh, 0)
    If fsize < 128 Then
        CloseHandle fh
        Exit Function
    End If
    'The ID3v1 tag is located 128 bytes from the end of the file, so look at this position.
    SetFilePointer fh, fsize - 128, 0, FILE_BEGIN
    ReadFile fh, RdTag, Len(RdTag), 0, ByVal 0
    CloseHandle fh

    If Data2String(VarPtr(RdTag.Identifier(0)), 3, ENC_ISO) = "TAG" Then
        ResetTag outTag

        'An ID3v1 tag is present.
        outTag.Artist = Trim$(Data2String(VarPtr(RdTag.Artist(0)), 30, ENC_ISO))
        outTag.Album = Trim$(Data2String(VarPtr(RdTag.Album(0)), 30, ENC_ISO))
        outTag.Title = Trim$(Data2String(VarPtr(RdTag.Title(0)), 30, ENC_ISO))
        If RdTag.Comment(28) = 0 And Not RdTag.Comment(29) = 0 Then
            'ID3v1.1 tag: byte 28 of the comment tag is zeroed, byte 29 is the track number.
            outTag.Comment = Trim$(Data2String(VarPtr(RdTag.Comment(0)), 28, ENC_ISO))
            outTag.TrackNr = RdTag.Comment(29)
        Else
            'ID3v1.0 tag: contains no track number.
            outTag.Comment = Trim$(Data2String(VarPtr(RdTag.Comment(0)), 30, ENC_ISO))
        End If
        outTag.Genre = GetGenreName(RdTag.Genre)
        outTag.SongYear = Trim$(Data2String(VarPtr(RdTag.SongYear(0)), 4, ENC_ISO))

        ReadID3v1 = True
    End If
End Function

Public Sub Read_File_Tags(sFile As String)
 Dim TAGv1 As ID3Tag
 Dim TAGv2 As ID3Tag
 
 
 m_sFileName = sFile
    
 If Dir(m_sFileName) = "" Then Exit Sub
    
   'Reset the local tag structure, and read the ID3 v2 tag.
   ResetTag myTag
   myOtherinfo.Lyrics = ""
   myOtherinfo.Images = ""
   
   ReadID3v2 sFile, TAGv2
   
   '// Read ID3V1 Tags
   ReadID3v1 sFile, TAGv1
   
   myTag.Title = TAGv2.Title
   myTag.Artist = TAGv2.Artist
   myTag.Album = TAGv2.Album
   myTag.SongYear = TAGv2.SongYear
   myTag.Genre = TAGv2.Genre
   myTag.Comment = TAGv2.Comment
   
   
   If TAGv2.Title = "" Then myTag.Title = TAGv1.Title
   If myTag.Title = "" Then
      myTag.Title = Right(sFile, Len(sFile) - InStrRev(sFile, "\", , vbTextCompare))
      myTag.Title = Left(myTag.Title, Len(myTag.Title) - 4)
   End If
   If TAGv2.Artist = "" Then myTag.Artist = TAGv1.Artist
   If TAGv2.Album = "" Then myTag.Album = TAGv1.Album
   If TAGv2.SongYear = "" Then myTag.SongYear = TAGv1.SongYear
   If TAGv2.Genre = "" Then myTag.Genre = TAGv1.Genre
   If TAGv2.Comment = "" Then myTag.Comment = TAGv1.Comment
   
    '// buscar la letra de la rola y otra infomacion
    HasLyrics3Tag = GetLyrics3Tag(sFile, myOtherinfo)
             
    If m_fReadMPEGInfo = True Then
       ReadMPEGInfo sFile, myMPEGInfo
    End If
  
End Sub

Public Sub Write_File_Tags()
On Error Resume Next
     If Dir(m_sFileName) = "" Then Exit Sub

     WriteID3v1 m_sFileName, myTag
    
    'The WriteID3v2 function can be passed a buffer pointer as parameter. See the comment
    'at the function implementation for details.
     WriteID3v2 m_sFileName, myTag, VERSION_2_3, False
     
     WriteLyrics3Tag
End Sub

Private Function WriteLyrics3Tag() As Boolean
   
   Dim WholeTag As String
   Dim TagSize As String * 6
   Dim Position As Long
   Dim MoveMP3Tag As Boolean
   Dim fn As Integer
   Dim UseOldInfo As Boolean
   Dim NewLyr As Boolean
   Dim fh As Long, fsize As Long
   Dim RdTag As v1Tag
   Dim bID3v1Tag As Boolean

   
   WholeTag = ""
   
   On Error GoTo hell
      
   If myOtherinfo.Author <> Empty Then NewLyr = True
   If myOtherinfo.OtherInfo <> Empty Then NewLyr = True
   If myOtherinfo.Lyrics <> Empty Then NewLyr = True
   If myOtherinfo.Mood <> Empty Then NewLyr = True
   If myOtherinfo.Quality <> Empty Then NewLyr = True
   If myOtherinfo.Situation <> Empty Then NewLyr = True
   If myOtherinfo.Images <> "" Then NewLyr = True
   
   'build the tag
   If NewLyr = True Then
       fh = CreateFile(m_sFileName, GENERIC_READ, FILE_SHARE_READ, ByVal 0, OPEN_EXISTING, 0, 0)
        If fh = INVALID_HANDLE_VALUE Then Exit Function
        fsize = GetFileSize(fh, 0)
        If fsize < 128 Then
            CloseHandle fh
            Exit Function
        End If
    
        'The ID3v1 tag is located 128 bytes from the end of the file, so look at this position.
        SetFilePointer fh, fsize - 128, 0, FILE_BEGIN
        ReadFile fh, RdTag, Len(RdTag), 0, ByVal 0
        CloseHandle fh

        If Data2String(VarPtr(RdTag.Identifier(0)), 3, ENC_ISO) = "TAG" Then
            bID3v1Tag = True
        Else
            bID3v1Tag = False
        End If

      WholeTag = "LYRICSBEGIN"
      Dim IND As String
      'lyrics
      If myOtherinfo.Lyrics <> "" Then
         WholeTag = WholeTag & "LYR" & Format(Len(myOtherinfo.Lyrics), "00000") & myOtherinfo.Lyrics
      End If
      'other info
      If myOtherinfo.OtherInfo <> "" Then
         WholeTag = WholeTag & "INF" & Format(Len(myOtherinfo.OtherInfo), "00000") & myOtherinfo.OtherInfo
      End If
      'author
      If myOtherinfo.Author <> "" Then
         WholeTag = WholeTag & "AUT" & Format(Len(myOtherinfo.Author), "00000") & myOtherinfo.Author
      End If
      'images
      If myOtherinfo.Images <> "" Then
         WholeTag = WholeTag & "IMG" & Format(Len(myOtherinfo.Images), "00000") & myOtherinfo.Images
      End If
      'quality
      If myOtherinfo.Quality <> "" Then
         WholeTag = WholeTag & "QUA" & Format(Len(myOtherinfo.Quality), "00000") & myOtherinfo.Quality
      End If
      'situation
      If myOtherinfo.Situation <> "" Then
         WholeTag = WholeTag & "SIT" & Format(Len(myOtherinfo.Situation), "00000") & myOtherinfo.Situation
      End If
      'mood
      If myOtherinfo.Mood <> "" Then
         WholeTag = WholeTag & "MOO" & Format(Len(myOtherinfo.Mood), "00000") & myOtherinfo.Mood
      End If
            
      'calcuate the size
      TagSize = Format(Len(WholeTag), "000000")
      'append the end identifier
      WholeTag = WholeTag & TagSize & "LYRICS200"
   End If
   'prepare for writing
   fn = FreeFile
   Open m_sFileName For Binary As #fn
   
   'set to just before the current id3 tag if has.
   If bID3v1Tag = True Then
      Position = LOF(fn) - 127
   Else
      Position = LOF(fn) + 1
   End If
   
   'if there is a Lyrics3 tag, then go back to the beginning of the old one
   If HasLyrics3Tag Then Position = Val(PosTagLyr)
   
   ' write the lyrics3 tag if there is one...
   If WholeTag <> Empty Then
      Put #fn, Position, WholeTag
      Position = Seek(fn)
   End If
   'set the last byte of the file
   Position = Seek(fn) - 1
   Close
   'make sure this is the end of the file which is needed if this tag is smaller than the old tag.
   SetFileLength m_sFileName, Position
      
   Exit Function
hell:
End Function

Private Sub SetFileLength(ByVal filename As String, ByVal NewLength As Long)
   'Will cut the length of a file to the length specified.
   
   Dim hFile As Long
   Dim l As Long
   Dim lpSecurity As SECURITY_ATTRIBUTES
   'if file is smaller than or equal to requsted length, exit.
   If FileLen(filename) <= NewLength Then Exit Sub
   'open the file
   hFile = CreateFile(filename, GENERIC_WRITE, ZERO, lpSecurity, OPEN_EXISTING, FILE_FLAG_BACKUP_SEMANTICS, 0)
   'if file not open exit
   If hFile = -1 Then Exit Sub
   'seek to position
   l = SetFilePointer(hFile, NewLength, ZERO, ZERO)
   'and mark here as end of file
   SetEndOfFile hFile
   'close the file
   l = CloseHandle(hFile)
End Sub


Private Function WriteID3v1(ByVal filename As String, ByRef inTag As ID3Tag, Optional ByVal bIgnoreReadOnly As Boolean = True) As Boolean
    Dim fh As Long, fsize As Long
    Dim rdBuf(2) As Byte, WrTag As v1Tag

    If bIgnoreReadOnly Then IgnoreReadOnly filename
    fh = CreateFile(filename, GENERIC_READ Or GENERIC_WRITE, FILE_SHARE_READ, ByVal 0, OPEN_EXISTING, 0, 0)
    If fh = INVALID_HANDLE_VALUE Then Exit Function
    fsize = GetFileSize(fh, 0)

    'Check if there's already an ID3v1 tag present.
    'The ID3v1 tag is located 128 bytes from the end of the file, so look at this position.
    If fsize >= 128 Then
        SetFilePointer fh, fsize - 128, 0, FILE_BEGIN
        ReadFile fh, rdBuf(0), 3, 0, ByVal 0
        If Data2String(VarPtr(rdBuf(0)), 3, ENC_ISO) = "TAG" Then
            'An ID3v1 tag is present.
            SetFilePointer fh, fsize - 128, 0, FILE_BEGIN
        Else
            'An ID3v1 tag isn't present.
            SetFilePointer fh, fsize, 0, FILE_BEGIN
        End If
    Else
        'An ID3v1 tag isn't present.
        SetFilePointer fh, fsize, 0, FILE_BEGIN
    End If

    'Build up writing tag.
    String2Data "TAG", 3, VarPtr(WrTag.Identifier(0)), False
    String2Data inTag.Artist, 30, VarPtr(WrTag.Artist(0)), False
    String2Data inTag.Album, 30, VarPtr(WrTag.Album(0)), False
    String2Data inTag.Title, 30, VarPtr(WrTag.Title(0)), False
    String2Data Replace$(inTag.Comment, vbNewLine, " "), 28, VarPtr(WrTag.Comment(0)), False
    'ID3v1.1 tag: comment(28) = 0, comment(29) = track number.
    WrTag.Comment(28) = 0
    WrTag.Comment(29) = 0
    If IsNumeric(inTag.TrackNr) And Len(Trim$(inTag.TrackNr)) <= 3 Then
        If CLng(inTag.TrackNr) > 0 And CLng(inTag.TrackNr) <= 255 Then
            WrTag.Comment(29) = CByte(inTag.TrackNr)
        End If
    End If
    WrTag.Genre = GetGenreIndex(inTag.Genre)
    String2Data inTag.SongYear, 4, VarPtr(WrTag.SongYear(0)), False

    'Write the tag to file.
    WriteFile fh, WrTag, Len(WrTag), 0, ByVal 0

    CloseHandle fh
    WriteID3v1 = True
End Function

Private Function DeleteID3v1(ByVal filename As String, Optional ByVal bIgnoreReadOnly As Boolean = True) As Boolean
    Dim fh As Long, fsize As Long
    Dim rdBuf(2) As Byte

    If bIgnoreReadOnly Then IgnoreReadOnly filename
    fh = CreateFile(filename, GENERIC_READ Or GENERIC_WRITE, FILE_SHARE_READ, ByVal 0, OPEN_EXISTING, 0, 0)
    If fh = INVALID_HANDLE_VALUE Then Exit Function
    fsize = GetFileSize(fh, 0)
    If fsize < 128 Then
        CloseHandle fh
        Exit Function
    End If

    'Check if there's an ID3v1 tag present.
    'The ID3v1 tag is located 128 bytes from the end of the file, so look at this position.
    SetFilePointer fh, fsize - 128, 0, FILE_BEGIN
    ReadFile fh, rdBuf(0), 3, 0, ByVal 0
    If Data2String(VarPtr(rdBuf(0)), 3, ENC_ISO) = "TAG" Then
        'An ID3v1 tag is present, remove it.
        SetFilePointer fh, fsize - 128, 0, FILE_BEGIN
        SetEndOfFile fh
    End If

    CloseHandle fh
    DeleteID3v1 = True
End Function

Private Function GetLyrics3Tag(sFile As String, ByRef outOtherInfo As tOtherInfo) As Boolean
 On Error GoTo hell
   Dim Position As Long
   Dim FieldData As String   '// save Text of field
   Dim FieldID As String * 3 '// save Field ex: LYR, ALB, AUT... etc...
   Dim LengthField As String
   Dim Size As Long          '// size of File
   Dim tagtype As String * 9 '// save end tag lyrics
   Dim Byte11Buffer As String * 11 '// save LYRICSBEGIN
   Dim Byte5Buffer As String * 5   '// Length of Field
   Dim lEndLyr As Long
   Dim LyricEndID As String * 6
   Dim fh As Long, fsize As Long
   Dim RdTag As v1Tag
   Dim bID3v1Tag As Boolean

    fh = CreateFile(sFile, GENERIC_READ, FILE_SHARE_READ, ByVal 0, OPEN_EXISTING, 0, 0)
    If fh = INVALID_HANDLE_VALUE Then Exit Function
    fsize = GetFileSize(fh, 0)
    If fsize < 128 Then
        CloseHandle fh
        Exit Function
    End If
    
    'The ID3v1 tag is located 128 bytes from the end of the file, so look at this position.
    SetFilePointer fh, fsize - 128, 0, FILE_BEGIN
    ReadFile fh, RdTag, Len(RdTag), 0, ByVal 0
    CloseHandle fh

    If Data2String(VarPtr(RdTag.Identifier(0)), 3, ENC_ISO) = "TAG" Then
      bID3v1Tag = True
    Else
      bID3v1Tag = False
    End If
    
    '//reset size of tag
   LengthField = "000000"
   
   '//open the file
   fh = FreeFile
   Open sFile For Binary As #fh
   'get filesize
   Size = LOF(fh)
   
   'get the tag END
   If bID3v1Tag = True Then
     'get the tag END [ (127 -> ID3v1Tag) + (9  -> LYRICS200) ] = 136
     lEndLyr = Size - 136
     Get #fh, lEndLyr, tagtype
   Else
     lEndLyr = Size - 8
     Get #fh, lEndLyr, tagtype
   End If
   
   'if tag is valid then
   If tagtype = "LYRICS200" Then
      
      'get the size of the tag  ( 136 - 6 ) = 142
      Get #fh, lEndLyr - 6, LengthField
      
      'set the position to the first byte of Lyrics
      Position = lEndLyr - 6 - Val(LengthField)
      
      'get the beginning of tag
      Get #fh, Position, Byte11Buffer
      'save beginning lyrics tag
       PosTagLyr = Position
       
      If Byte11Buffer <> "LYRICSBEGIN" Then
         'invalid Lyrics3 version 2 tag! we don't support version 1...
         Close
         Exit Function
      End If
      
      'first field ( Position + LYRICSBEGIN )
      Position = Position + 11
      
      'keep getting fields until we get to the end of the tag
      Do Until Position >= lEndLyr - 9
      
         'the field id -> LYR-ALB-AUT. etc...
         Get #fh, Position, FieldID
         
         'the size of the field
         Get #fh, Position + 3, Byte5Buffer
         LengthField = Val(Byte5Buffer)
                  
         'make room for the data
         FieldData = Space(Val(LengthField))
         'get the data
         Get #fh, Position + 8, FieldData
         'and fill the approprate field
         Select Case FieldID
            Case "LYR"
               outOtherInfo.Lyrics = Trim$(FieldData)
            Case "INF"
                outOtherInfo.OtherInfo = Trim$(FieldData)
            Case "AUT"
              outOtherInfo.Author = Trim$(FieldData)
            Case "EAL"
               outOtherInfo.Album = Trim$(FieldData)
            Case "EAR"
               outOtherInfo.Artist = Trim$(FieldData)
            Case "ETT"
               outOtherInfo.Title = Trim$(FieldData)
            Case "IMG"
               outOtherInfo.Images = Trim$(FieldData)
            Case "QUA"
               outOtherInfo.Quality = Trim$(FieldData)
            Case "SIT"
               outOtherInfo.Situation = Trim$(FieldData)
            Case "MOO"
               outOtherInfo.Mood = Trim$(FieldData)
         End Select
         
         'now set the postion to the beginning of the next field
         Position = Position + 8 + Val(LengthField)
      Loop
      'set the flag
      GetLyrics3Tag = True
   End If
   
   Close

   Exit Function
hell:
   'close all open files
   Close


End Function



'**********************************ID3v 2.2, 2.3, 2.4****************************************

Private Function ReadID3v2(ByVal filename As String, ByRef outTag As ID3Tag) As Boolean
    Dim fh As Long, fp As Long, TagFooter As v2TagHeader
    Dim rdBuf(3) As Byte

    fh = CreateFile(filename, GENERIC_READ, FILE_SHARE_READ, ByVal 0, OPEN_EXISTING, 0, 0)
    If fh = INVALID_HANDLE_VALUE Then Exit Function

    'Search for an ID3v2 tag.
    Do
        SetFilePointer fh, fp, 0, FILE_BEGIN
        ReadFile fh, rdBuf(0), 3, 0, ByVal 0
        If Data2String(VarPtr(rdBuf(0)), 3, ENC_ISO) = "ID3" Then
            'We've got an ID3 v2 tag.
            fp = ReadV2Tag(fh, fp, outTag, False)
            ReadID3v2 = True
        Else
            fp = -1
        End If
    Loop While Not fp = -1

    If Not ReadID3v2 Then
        'No tag found at the beginning of the file.
        'Version 2.4 allows the ID3 tag to be appended to the audio data. So, look if we can
        'find such a tag, identified by a footer. According to id3.org, the ID3v2 footer should
        'be added before other tagging systems, therefore prepended to an possibly existing ID3v1 tag.

        fp = GetFileSize(fh, 0)
        SetFilePointer fh, fp - 128, 0, FILE_BEGIN
        ReadFile fh, rdBuf(0), 3, 0, ByVal 0
        If Data2String(VarPtr(rdBuf(0)), 3, ENC_ISO) = "TAG" Then
            'Look before the ID3 v1 tag.
            fp = fp - 128 - Len(TagFooter)
        Else
            'Look at the end of the file.
            fp = fp - Len(TagFooter)
        End If
        If fp < 0 Then fp = 0

        SetFilePointer fh, fp, 0, FILE_BEGIN
        ReadFile fh, TagFooter, Len(TagFooter), 0, ByVal 0
        If Data2String(VarPtr(TagFooter.Identifier(0)), 3, ENC_ISO) = "3DI" Then
            'Got a footer. Calculate the header position from the size.
            fp = fp - Data2Long(VarPtr(TagFooter.Size(0)), True) - Len(TagFooter)
            If Not fp < 0 Then
                ReadV2Tag fh, fp, outTag, False
                ReadID3v2 = True
            End If
        End If
    End If
    CloseHandle fh
End Function

'Returns the file pointer for the next tag, or -1 if there is no tag appended.
Private Function ReadV2Tag(ByVal fh As Long, ByVal fp As Long, ByRef RdTag As ID3Tag, ByVal bToMem As Boolean) As Long
    Dim TagSizes As v2TagSizes, arrp As Long
    Dim TagHeader As v2TagHeader, TagHeaderFlags As v2TagHeaderFlags
    Dim ExtHeader23 As v2_3ExtHeader, ExtHeader24 As v2_4ExtHeader
    Dim FraHeader22 As v2_2FrameHeader, FraHeader234 As v2_34FrameHeader
    Dim FraDataSize As Long, FraUnsynchDecr As Long, fraID As String, FraText As String
    Dim FraOffset As Long, bSkipFra As Boolean
    Dim TagData() As Byte, SizeBuf(3) As Byte


    ReadV2Tag = -1

    '------------------------------------------------------
    '********************* TAG HEADER *********************
    '------------------------------------------------------

    SetFilePointer fh, fp, 0, FILE_BEGIN
    ReadFile fh, TagHeader, Len(TagHeader), 0, ByVal 0
    If TagHeader.Version(0) < 2 Or TagHeader.Version(0) > 4 Then Exit Function
    'The size stored in the header ex    'Version   (er.Versioim TagData() As Byte, SizeBuf(3g, or -1 if Buf(3g, or -1 if Buf(3g, ofrom r -1 d in the header ex    ETT"
   ing(VarPtr(Rdo.rinfieldDxt As String  
   Dic   EFunct
 a$ CreateFile(filename, GENERIC_READ, FILE_SHARE_READ, ByVal 0, OPEN_EXISTING, 0,ar TagHeade$ CreateFi stored in thNERIC_READ, FILE_SHARE_READ, ByVal 0Header
    Dim Fr. TagHeade$     Dim Fr. TagHeade$    ,Lion <> "" Then
pEnd Function

P Long, TagFooteko    ETT"
   ing(VarPtr(Rdo.rinfieldDxt As STAGv2.Artist = "" Then myl 0, OPEN_EXISl 0, OPEN_EXISl 0, Sl 0, OPEN_EXISl 0, Sl 0, 0, OPEN*********bhEgstored in thNERIC_READ,     Long the end of the file,ion

Public rebhe end]d in ong, TagFooteko  ctio'aaaaaaaaaaaaaaaaaaaaaaaaaaaaaSub
   '_READ,     Long 
   '_READ,     Long 
   '_READ,     Long 
   '_READ,     Loea1FTrue
End FunctiondOnsize.
       L)Eored in th.irue
Enoy    IN
 ong, lpNu3e:
    ExtenFILE_SntiondOnsizeal bTo   
PubliaTagHe.rinfi     bIDthe header position from the size.
      header position from the size(
     lEndLyoteko  iooteko Public size.
  ----------------------        fp = ReadV2Tag(fh, fp, outTag, False)              bhEgstored in,ag, False)              bhEgstored ,ag, sizeal bToo)adFile fh, rdBuf(0MOO" &   
   '//open the file
   fh = FreeFile
d'iNe
d'iN= 1
    EN, sizeal bToo)adFile fh,ByVal 0Heade= WholTrsIf
   'set the la  hsla  hsla  hsla  hsla  hsla  hsla  hsla  hsla  hsla  hsla  hsla  hsla  hsla  a As Boolean
   =ion = -1 Thenhsla  hsla  hsla  hsla  hsla  hsChsla xn =the siz'L(WrTag.Comment(0)), Fa      o ,ag, si_REAo ,ag, siolTrsIf
   'set the la  hsla  hsla  hsla  hsla  hsla  hsl     header po2  Comment (Onsizeal bTag,=t #fhBsiolTrsIJi    end herIeadOnly As Boolean = TrFooter.Size(0)),Lyr oter.Se
   Size g 
   '_ROnly As Booa  hsla  hsla la  hsla  hsWoVa  hsla  hsla  a As B    g fLong
    Dim TagHI'//open terPtr(Rdo.rinfle, s- ValScan#Cabaret#New ong, lpNu3e:
    ExtR
"Eurtr(Rdo.rinfle, s- Val#Disco#Fun  'ThevteFi store*FILE_BEGIN
        ReadFse
    End If .rinf"Eurtr(Rd2sDim TagHI'//o.oI'//o.oI'//o.oI'//o.oI'//= Position + 8 + Val(LengthField2l(E_ATnfsize = GetFileSize(fh, 0)oolean = Tean
   =ion =#Cabaret#New'Fgstored ,sperNot ReadID3v2 Then
 stored ,uhFun  'ThevteFi stoaaadID3ing)
uhFu---erNot    Co 
   ' wriExtR
"Eurtr(Rdo.T= TrifIN" Tl        outOtherInf=l      CreateFile(Trip-Hop#Vocbum
 t70r(Rdo.T= TrifIN" Tl        ou.vr(rdBuf(0))roperty Get MPEG_Emphasis(g, lpBufferAmment(29 the tl Booa  hsla  hsla la  hslsiti 0,   '//open tMe ID3v2 Fu---Up#Vocbum
 t7sla  hsla  hsChsla xn =the siz'L(WrTag.Commen,cbum
 t7sla  hsla  hsChsla xn =the siz'L(Wrarivate Declare Function teFiIN" Tlate Declare  -Hop#V  ID3v2VersionVal fp As Lon 'ThevteFi stoaaadIDtion

P Lono
  hsla  hsChsla xn =the stion

P La", 4, Vnter fh, fp, 0, FILE_BEGINk#Folk#" & _
"Folk-Rock#Nationig, tn,cbum
 t7sl.Commensla  hsla  hsla    d2l(Eu    ou.vr(rdBhFu---erNoteFirtiss() As Strin-erNo.Eu    

P La",s Strr oter = TrFoot&Val fp As Lon 'ThevteFi stoaaadIDtgs
    Dim Ek#Euro-Techtems, there         Caseac, Fxn
 strtem      an .Artistd]d in ong, TagFooteko  ctio'f Data   an .Artistd]d ini     Casea      
      I0Header
   uhen
       eldID
          d.SongYear
E1c, FePointer fh,xn =th       d.Son fh,xn =th       daLon 'ThevteFi M
               outOtherInfo.ah1GCtherInfo.6w As String)
=th  w As 
 strtem      ac5adID3v2 Then
  L)Eored iQ 2.3, 2.4**********************    6ReSePoin


'**********eion 1...   'ap2.3, 2.4*ject AfinTaI4*ject AfinTalU"Eurtr(Ra

'**********eion 1IDtgs
    Dim Ek#Private Declare Function ReadFaM'Thevt.4****************** 3rFlags As v2TagHeaderc    ou.vr(rdBhFu---erNoteFirtiss()fhOtherInfo>lags As v2TagHeaderNoteFirtissToMem As Booleaned inderFlags As v2TagHeadnVal fp As L8BooleanedECu---erNoteFiLongrfeHeaocitrr oter = TPleanedECu--m       oteFiLonogs As v2TagHeaderNoteFirtissToMem rinfo.OtherI    Loea1FTrue
7adnVal fp As L8BooleanedECu---er'/tOtherI6oBt of a file I6oBt  oufeHeaocitrr oter =Eored red iY I6oBeBhFuREAD,     Loeion 1FiissToMem rinFirtissEorof th9laa)
         m,   
sis(g, lpBufferAmment(29 the tags Bool         m,   
sis(g,csToMemte FunctnrinFirtissEo.org, Unss(g, lpBufferAmment(     m2.4*ject AfinTaI4xn
 strtem      an .Artistd]d in ong, TagFooteoteFirtissTo of the nafeHeaocitrr oteFile Lib "kernel32" AliadID3v2 <= 255 Th    m2.   
sisEurt'ErAmmen 6
  edECu---er'/tOtherI6oBt of a file I6oBt  oufeHeaocitrr oter =Eored red iY I6oBeBhFuREAD,     Loeion 1FiissToMem rt of a file SByRef m rtU2.3,e7fferred iY I6c" Tl        oug, lpBufferAmment(u.Other4I'////////////////////////////////////et Mood() Ae postion too//////V  ID3v2VersionVam/ I6oBt eila  hsl     headern ID3 v2 tPrivather4I'//////   ExtR
"Eurtr(Rg.ThevteFi MO(g, l m rte Lib "kernel333333me local tag G HEADER *me As e Lib "kernel333333me locald iY I6c" Tl        (Rg Prope_
oI'/eTag As SS: contai,ity <> Empt(ro-Techtems, there         CaPEN_EXIST EN,  'set th Field
   Dim lEndLyr AsrtissToMem rinfoFunction
0, OPEal tag G HEARong
 (WrTag.Commen,cbum
 t7sla  9(WrTag.anction
0, O
"Eurrrrrrrrrrrf<    End If
      'ges    (Rg Pla xn =the stion

P LWrTag.anction
0, O
sAs SS:tle = ""ity -Techtemss As Long, ByVal OnNstion

P LWrTag.anctnction
0, O
sAs SS:Fa   eadtemss As Long, ByVal OnNsoublic AGv2.Commemment (Onsizeal bT7'Kze(fh, 
Public Prnctnction
0, O
sAs SS:Fa   eadtemss As Long, ByVal OnNsoublic AGv2.Commemment (Onsizeal bT7'KBeadg.ancnctnction
0, O
sAs SS:Fa
End FuncyVal dOg is loY////V  ID3v2Ver.anctnction
0, O











Too)adFile 33Too)adLib "kuncyV= TrifIN" Tl        outOtherInf=l      CreateFile(Trip-Hop#Vocbum
 t7tWR    Get   Id'bui.Title = Left(myT'  End I ByValal OnNsoublic AGv2arPtr = -1 Teate)NoteFirtiss.
0, O
sAsder StrLr StrLx  ou////V  ID3v2Ver.ancloY////V  ID3v2Ver.anctnpi() As Strini"MOO" & Format(Len(m YSh n(m YSh n(l32" Alias tnction
0,U'Oo hell
  ******

Private Function ReadID3
7a" Alias_ eadID3
leSiDDDDDDDs_ eadID3
leSiDDDDDDDs_omme.vm
End PropereSiDDDDDDDWe_
o:Fa
End C_WRITE, FILE_SHARE_READereSiDDDDDDDunction ReadID.oublic AGv2arPtr =S:Fa   eadtemss As L    2Exit Function
    fsize = GetFileSize(fh, 0)
    If fsbum = ze(fon 2 tre.vm
End1 ong, lpNu3 R
"EuYY
"EuYY
"EuYY
"lE, FILEUl(LengthField2l(E_>JsI'/eT6nlion(0) < 2b2DT6npxivate FuncS2" Alias t"5Size(f.Size(tiR"ISO) = "TAGNject Co 
   ' wriExtRE_READe0, O
n
    fsize - 128, 0, FILE_BEGIN
    ReadFile ue
 DT6npxi



L"
         arPtr( ing
    Comment = myTag.Comment
End Property

Public Property Let AlbuS0, O
n
    fsiz( arPtr( i2TING, 0ty Let AlbuS0, O
n
    fsiz
n
g appended.
Pri5a  w As.bum
 t7sla  hsla  G, 0tAll
TING, 0tag G HEAD\Formarm rte Lib "kernel333333me local tag  fsiz
n
g app2sata() As um = ze(fon 2 tuuirty
hction
hell:
End Fn-erNoe file, so L     
     
     soe file, so G, 0,eommens///   ExtR
"Eurtr(Rg.ThevteFi MO(g, l m rtommens///   ExtR
"Euas tnction
128 _Emphasis(g, lpBufu.vr(rdBuf_sFileNam=fdBuf_s ExtR
' l m rtommens//d of tFileNaaaaa    Casea  
' l m rtomme of arty

P*****o)
  A(end]d in ong, TaLeft(myT'  E
  A(en0/   ExtR
"Euas tnction
128 _Emphasis(g, lpBufu.vr(rdBuf_sFileNam=fdBuf_s ExtR
' l m rtommeMeSoter fh, f       arPtr(c6I0, O
Sze = GetFildData)
   gs As v2TagHeadnVal fp AExit Funct O
Sze a  hsl     tsze a 2.3,esSze a  hsl     tsze a 2.3,0KExtR
' l m rtommeMeeackNr(ByVal Nee postion too//////V  ID3v2VersionVam/ ta)
teFi ' l m rtommeMeeackNr(ByVal Nee postion t"Y
"EuYY
"lE, FILEUl(LengthFiel Close6Size = LOF(fh)
 
Too)adFile 3= INVALIDa2Stpostion t"Y
 'images
 oV  IDSPropert2uf_sFileNam=fdBuf_seeac t.
        ktpos Diw.
        'Ve LOF(fh)
 
Too)adFiltpos Diw.
        'Ve LO
 
Too)adFiltpos Diw.pfsiz------------------        fp = R1s
     
     soe file, so G, 0,eommens///   ExtR
"Eurtr   ktpoUs///   ExtR
--        fp = R1s
     
     Ud If
      'p = R1s
     
     Ud If
  e - 128, 0, FILE_BEGIN
    R
    Sa  
   s
 saength, ZILE_saength, ZeFile(filename, GENERIC_R,eommens///   Ext5ihe comme    arPtr( ing
      bhEgstored in,ag, False)              bhile I6oBt ens///   Ext1y GetFileSize(fh,hen Exit    fp = R1s
     
          bhile Io- 128, 0, FILE_BEGIN
    R
    Sa  e() As String
 1EGIN
    R
    Sa  
   s
 saength, ZILE_saength, ZeFile(filename, GENERIC_R)9rsionVam/ ta)
teFi ' l m rtommeMeeackNr(ByValadF  'set th Field
   Dim lEndLyr AsrtissToMem rinfoFunctio,sssToMem rir
     
ia  e() As String
be//////ARE_READereSiDDDDDDDunction Reem rinfoeFi 'v  
ia  e() As 
ia  3= INVALIDa2Stpostion tsetFildData)
   gs As v2TagHe 
   '4 'v  
ia  ,hen E la  hsla  hsla  hsla  hsla  hsla  hsla  hsla  hsla  hsla  
   gea3
        gea3
  eMeeackNr(B   R
    Sa  
  EN,  'set th Fiaong, ByVal OnNsoublic AGv2.Commemment (Onsizeal bT7'Kze(fh, 
Public Prnctnction
0, O
ublic AGv2.ClSOnsizeal bT7'Kze(fh, 
Public Prnctnctio LO
 
Too)adFiltpos Diw.pf'set th Fiaong, Sion
0W1.Identifv  
ia  e() As 
ia  3= INVALIDa2Stpostion tsetFildData)
   gs As v2TagHe 
   '4 'v  
ia  ,hen E la  hsla  hsla  hsla  hsla  hsla  hsla  VILEUl(LengthFiel-ee peTAGv1 As waFiltposam/ --m       oteFiLo
   xction Re"engthF ZeFile(filename, 2.ClSOnsizeal bT7'Kze(ostion too//////V  ID3v2VersionVam/ I6oBt eila  hsl     headern ID3 v2 tPrivather4I'//////   ExtR
"Eurtr(Rg.ThevteFi MO(g,b8, 0, FILE_BEGIN6oBd.
Pri5a     If Dir(R *mv  zeal name  look at Info.ID3v1Versioniiiiiiiiiiiiiiiii0EncodedBy =ion
    he siz'L(WrTaone"
k too/g too/g too/g e si0 And Not RdTammens///   ExtR
"Eurtr   ktpo 1N6o(CRC
End Property
Public aterjrackEXIST EN,  'set th FielCuoiolic aterjra, Sion
0W1.iYY
"EuYY
"EuYY
"lE, FILEUl(LengthFNsoublic AGv2 hsl_r3 v2 tPrivion
0ield
1 Teate)NoteFirtiss.
0, O
sAsder StrLr StrL StrLr StrL StrLr StrStrL Tla  hsla))
        outTag.Album aing
 1EGIN
   IData)
   gs As v2TagHeadnVal fp Aeac t.
        ktpos Diw.
        'Ve LOF(fh)
 
Too)adFiltpos Diw.
 field id -> LYR-ASion
0W1.IdenAE/ I6RdTag As Id id -> LYR-ld id ->Sa  
   ssssssssssHClosesl_rfh)
 D -> LoMem rir
     
ia  e() As String
ssHClosesl hsla  hsh)
 D -> d -> LYR-M1  Prnctnct herIw stoaaadf d -> LYR-M1  Prnctnct herIw stoaaa, lpBufferAmment(u.Other4I'//////(u.Othnct herIw stoaaa, lpBufferAmment(ur d -> kx7sla            ii   8 G, 0,eommens///   i  Get #fh, lrInfo Ang
sEorof th9laOtherIn , StrLr StrL StrLr StrL StrLr soublic AGv2 hsl_r3 v2 tPr(ur d -> kn i  Get #fh, lrInG HEAD\Formarm rte Lib "kernel333,
  _BEGIN
 a  
  EN,  ',somment  hsla  hsla Ha3
  ee Lib "kernel333iaong, ByVal OetrLr iAGv2.Comages
 oV  IDS
 a  
  EN,  ',sommeng, False)              bhEgstored ,ag'Kze(ostion <= 255 Th    m2.   
sisEurt'ErAmmen 6
  edECu---er'/tOtherI6oBt of a file I6oBt  oufeHeaocitrr oteeeHeaocitrr oteeeHeaocitrr oteeeHeatoaaa, lpB Readr4I'//////(u.Othnct hel tag  fsiz
tion <= 255 Th    m2.   
sisEurt'ErAmmen 6
  edECu---er'/tOtherI6oBt of a n
0, O
sAs SS:Fa   eadter'/tOtherI6oBt of a cS:Fa thField =Iw stoaaa, lpcSS:Fa   i   8 G, 0,eommenng
sEorof th9laenngen 6
55 Thadthsl   n-Eof t2oenre = itrr oteeeHea R
 N,  u=temso---er'/tOtherI6oBt of a n
0, O
s Boolean
    Dim TagDataenngen 6
55 Thar3 v2 s
   'close the file
   l = Clooaaa, lpcSS:FaVal
  eet3 v2 t cS: hsht3 v2a
'*********1edECu-'*********1edECu-'********If 
   
yTh    m2.   
sisEurt'ErAmmen itrr oteeeHeaocitt  oufTh    m2.   
liByte

 m2. 'make Dim TagDataenng, 0, 0)
    ILong 
   '_READ,t'ErAmmoe() As String
ssHClosesl hsla  hsh)
 D -> taenistd]d in ong, TagFootekag fim TagDatae
sisEurt'E -1 Teate)NotelYRICS lpBuaS in ong, TagFooOlop0l tag/edECu-'*aaa, lpBufferAmmeetFilePointer(hFi5   head bhiI6oBt ens///al ffla
         er(hFi5   head v2  
i StrL StrL(1ckNr) < 6
  edECu---er'/tOtherI6G HEADER */V  ID3v2Versio DDDDunr(h2TagHeaderc    ou.vr(ri_sFileB.DDDuat3 v2 t cS: hsht3 v2a
'*******TagHeadev2Versio DDDDunr(h'xs   bhEgs2G3,e7ffFa
End C_WRITE, FILE_S-1ng 
  C_WRITE, FILEeim TagDataenng, 0,G HEADER */V o
'*********1dECu---er'/tOthe3ADEC_WRIT.Eeim TagDat.inct gs As v2ar
   myTag.GYFa   6 */V  ID3v2Versio(
'*********1dECu---er'/tOthe3ADEC_WRIT.Eeim TagDat.inct gs As e7fAs v2ar
 DDDDunr(h2TagHeabc3ADEC_WRIT.Eeim TagDat.ERIC_READ, FILE_SHagDat.inct gsinct glByteITE, FILE_S-1ng 
m16G HEAo back to tpopnlionc3ADElr otepEC_WRf the file.
    
 ZILRIT.Eeim TagDat.inct gat.ERIC_READ, FILE_SHagDat.int of a n
0, O
s Boole  'Ve LOF(fhvOF(fhvOF(fhvOF(flocald iWo tpopnlionc3Ay = True
_PagFootekagttion Writet.int of a n
0, Pi4rInfo.Quality = adFile fh, rdBFootekag, rdBFof a n.Eei) Asd Properoperty Get fFile fh, rdBFooPent 
 D -> taenistd]d ierNoe file, sofer AM'4 'v  
ia  ,hen E la  hsla  hsla  hsla  hsla  hsla  hsla  VILEUl(LengthFiel-ee peTAGv1 As waFiltposam/ --m       oteFiLo
   xction Re"engthF ZeFile(filename, 2.ClSOnsizeal bT7'Kze(ostion too//////V  ID3v2VersionVam/ I6oBt eila  hsl     headern ID3 v2 tPriva As Long
la  , bIgnoreReaID_HANDLE_VaRo v2 tPrivaTrue
   
p writiaaaaaaYR-ASion
0W1.ss(g, lpBufferA1dECu--trL StrLr soublic AGv2 RtR
"YAmmeetFIla  hseE4ExtrsekagttioeY I6c" Tl        oug, lpBufferAmment(u.Other4I'///////////, 0, Fs5   tioeald 
sEorYAmmeetFIla  hnioeald 
sEo   2.ComagesOther2y////////, 0,pBufN  
p writiaaaaaaYR-ASion
0W1.ss(g, lpBuffeTtitiaaaaaaYR-ASion
0W1.ss(g, lpBuon tootOtherInf=  oug, 21ment (OnsizeaFeetFIla  hseE4ExtrgHe.rinfi     bAGet fme: If
gi5TS6npxivate FuncS2" Alias t"5Size(f.Size(tiR"ISOu.Other4I'//////
   
y     bAGet fme: If
gi5TS6npxivate FuncS2" Alias t"5Size(f.Size(tiR"ISTsL(fhvOF(flocaAlias t"5Size(f.Size(tiR"IHrInfo stionpxivate FuncS2" Alias t"5Size(f."5Size(f.Size(tiR"ISTsu As e7fAs v2ar Th    m2iNatio  look at I069r(tiR",ErAmiR",ErAmiTh    m2.Sion
0W1.ss(g, lpBufferrrrdthsltrL StrLr soublis1rAmiTh 6oBt of a filR",ErAmiist = Trim$(Do-ertion Writ2.Comtoot hsla  hwi=  
y     * t"5Si--trL Str Do-Get svstore*FILE:B Then
b E, FILEUl(S?
On Len(sFile) - InStrRev(s v2ar Th    m2iNat/V  ID3v.Commenl hsla , 0, ByVal 0
tiaaaDo-=eTag outsd P ID3 ar Tsllm_sFileName Th    m2iNat/V WStrRev(sllm_sFileName Th  sg) As OtFIla  hnir4ItiR",ErAmiR",ErT.Eeim TaD3 ar TstrRev(sllm_sFioir4ItiR",ErAmiR",ErT.dMoNG, 0te)3 v2 sILE:B The/////ARE_R
   mya  
   ssssmiR"2ar
ltpos Diw.g of tsInfo.6w As miR"ring, FraItiRimiR",ErT.Eeim TaD3 ar TstrRev(sllm_sFioir4ItiR",ErAmiR",ErT.dMoNG, 0te)3 v2 sILE:B The/////A.dMoNG  hnir4
p writiaEoBt eila  hsl   ILRIT.EmmeetFIla&
y     * t"5Si--trL9 fim TagDatae
sisEurt'Val OnNsoublic AGv2.Canctncti     v2ar Th    m2iNatio  Datae
FuREA$rdentifiem Tamiist = TrigDatae Data,ErT.Eeim TaD3 ar TstrRev(sllm_sFioir4ItiR",ErAmiR",ErT.dMoNG, 0te)3 v2 sILE:B The/////ARE_R
   mya  
   ssssmiR"2ar
ltpos Diw.g of tsInfo.6w As miR"rinong
la  , bIgnoreReaID_HANDLE_VaRo v2 p     As miR"rinoIssssssbSoir4ItiR<H(fhvOF(fhvOF(fhvO)adLib "kunmya  fh, fp, 0, FILE_BEGIN
    Rea
 DDDDunr(h2TagHeabc&IEndLyr - 6C Calculate the P0miR",ErT.dMoNG(fhvOF(flocsn't present.
Dent.
Dent.
DeTnr(h2EaT
   mya  
   sssp.dMoNG(fheMoNG(fheMoNG(fheMoNG(fheMoNG(fheMoNG(fheMoNG(fheMw2a  hi a buffer po(rtomment.
DeT mya  
 TrigDatatrL StData2String(TrigDa1fBheMoNG(fheMoR" & Form FePoif"LYRICSG(fheMiFILE_m TaD3 ar TstrRev(sllm_sFE_m TaD3 ar TsSG(fheMtrRev(sll      Else
w an ID3v2 tfheMtrRev(sll      Else
wngen 6
55 Thadthsl   n-Eosize <miR"r(fheMw2a  hi a buffer po(rtomment.
aFeetFIla  On Len(sFimya  
   sssp.Ila  )Z
Other2
  L)Eored iQ 2.3, 2.4**********************    6ReSePoin

heMw2EEEEEEEEEEEEEE3Ay = True
_PagFootekagttion ue
l8
55 Thadthsl   n-Eosize <m-Eosize <miR"r(fheMw2a <e4ItiR<ksll      Else
w an ID3v2 tfheMtrRev(sll      Else
wngen 6
55 Thadthsl   n-Eosize <miR"r(fheMw2a  hi a buffer po(rtomment.
aFeetFIla  On Len(sFimya  
   sssp.Ila  )Z
Other2
  L)Eored iQ 2.3, 2.4**********************    6ReSePoin

heMw2EEEEEeSePoin

heMw2EEEeMw2a gFIlasuof the  &UIla 6ReSePoePoin

heMw2emMoNG(fheMoNG(fheTagdmTaD3 ar TsSG(fheMtc=erAmme (fheMtc=erAmme (fheMtc=erAmme (fa    6ReSePoiid iQ 2.3, 2.4**********************    6R(fheMtc=erAmme (fhel4Fhel4Fhel4FWat(=eTag outsd P ID(fhe+++++++++++++++++++++++oASiQ 2.3, mme (fhel4Fhelsla  hsla  hsla  
   geSltpos DiwtatrL StDg, lpBufferr4ltposam/ la  hsla  hsla +++2hoNG( TsSG(fL StDg, ePoif"LYRICFtFi/ah, fsizeEEEEEEEE3Ay = Truum
   If TAGv2.SongDp writiaah, fsizeEEEEEEEE3Ay = TruW afeHeaocitTx,CUetFIla  On Le////mment(29) Function

PrivasDim TagH.Eeim TaD3 ar TstrReoIssssssbSoir4ItiR<H(fhvOFstrReoIsssssseim TaD3 ar TstrR,, 30, VC  OPoir5s(29) Lo
   xcti.Author
      End If
 TirRev(s v2a tim TaD3 ) As 
ia  3=wCFtF(As 
ia  3  6R(fheMtc=erAmme ((fheMtc=erAmme (v2ar Th    m2iNatItiR<H(fhvOFstrReoIsPMag ate the PlvOF(flocsn't preses///   Ext5ihe comme    arPtr( ing
      bSfSst

Ptr( ing
  lslaiagH.Eeim TaD3 ar TstrReoIssssssbSoir4ItiR teFiIN" Tla     End If
 TigiFiIN" Tla     End If
 Ti Sha(fhvOFst(
Prig
  lslaia
      Eg.Comm,iF Sha(fhvOF TAGv2ion

Privadle fh

        If Data2String(VarPtr(RdTag(VarPtr(RdT  If Data2Strm,iF ShEg.Comm,ito-=eTag outs Long
  3, 2.4****g
  3,(VarPtr(RI1gH.Eeim e*1edECu-'***Cr(RdT  If DataigiFiIN" X.2oa If Diar TstrRfhvOF)), Fa  hsla  hsla y AsfilL(*Cr(RdT  I2As 
ia  teFiIN" Tg   '// save Text                                            a  3=wCFtF(As 
ia  3  6R(fheMtc=erAmme ((ff
 TslaiagH.Eeim TaD3 ar TstrRO  Comment (OnsiiagH.tncti    h    eiR"IHrInfo stionpxivate F
           LLLLLLLLLLLLLLLLmnsAGNj<*1dECu---er,ErT.dMoNG, 0Fa  h'x 3=wCeFile(filela  hnioeald 
sEo   2.ComagesOthF ZeFile(filenahvOFstrReoIsPMag ate the PlvOF(flmpOth          hvOFstrReoIsPMag a   SetFarReoIsPMagl1
   IData)
   feHeaocitTpVal fp Aee F
  t field
   ing
      bSfSst

Ptr( ing
Eeim TagDat
     le fh

   cnd Not R*********ld
   pr( ing
      bSfSst

Ptr( i
   pr( ing
      bSfSst

Ptr( i
   pr( ing
      bSfSst

Pt   *ld
   pr( ing
      bSfSst
U**** TAG HEADER                                 -e bSfSst
U**** TAG HEADESfSst

PtSst
U**** TAG Sha(fhvOFst(
Pri**** TAG HEADESk at Info.ID3v1Versioniiii at InHooy,Xt3 v2 t cS:c=erAmme (fhel4Fhel4Fhel4FWat(=:c=er HEADERic aterjrackEP<H(fhvOFstInfo.I= Trim$(Do-ertion Writ2.Comtoot hsla  hwi=  
y     * t"5Si--trL Str Do-Get  FILE_BEG ssp.Ila  )Z
Other2
  L)Eored iQ 2.3c Komtoot hslaaaaaaaaaaaaaS, 0,ar TagHioniibli
 Ti Sha(fhTAG HEim TaD3 ar le fn
hell:
End Fn-erNoe file, so     t hsla  h_-'*a4FWat(=:c=er HEADERic aterjrackEibli
 Tij<*1dECu'*a4FWat(=:c=bli
 
  edECu--- ADERic aterjrackEP<H(fhvOFstInfo.Ih_-'*rPtr(Rat(=:c4 edECusXt3 volaiagpreerAmmoS:c=eac atu--- ADERic aterjrackEP<H(fhvOFsttion Writ2.Comtoot hsla  kEP<H2Amm3 volaiagprefle, s- Val# outTag       ReadFse
    End If .r (fh aterjracfPle(hFile)
End5Da2Stposti
y     * t"5M
  L)t2.Comto3 v2 t cS: hsht3 v2a
'*******o3 v2 t c
Prir Loea1End IIIII&i StrL StrL(1ckNr) < 6
  edECuar TstrReoIssssss(t c
PC-Eosize <m-Eosize <miR"r(fhe
   p(hedECuar TstrReoIssssse <m-EcfPle(hFile)   As miR"rinoI'on Writ2.Comtoot hsla  kEP<H2Amm3 volaiagpracfPlRic aterjracdFse
 ev2Ver    6ReSePoiid iQ 2.3, 2 hi aoNG(fheMoN
 ev2Ver TaD3 ar TstrR,, 30,42.3, 28f_s ExtR
 eadF 3, rackEP<H(Ur(fhe
 nfo.Ih_-'*rPtr(Rat(=:cIssssse <ml OnNsoublic AGv    'Tublic ext        Wnd ISst

ssss(m2iNat/V WStrRev(sllm_sFileName Th  sgeRic ahvOFstc eYRICSBEGIN"
  e() As 
i8f_s ExtR
 eadF 3, rH2AEP<H(Ur(, 2oot hsla        -e bSfSst
U**** TAG HEADESbLLLLL0)

    'CcL'ER  (EP<H(Ur(, h) As 
i8f_LVSoir4C HEADE29) Lo
  AEP<H(Ur'*a4Fc=er HEADER$5m2iNat/ , OPEN_EXckNr) < 6
  edECuar TstrR(mm3 vola
  edECuar TstrR(mtrLr soub   
sisE4edECoe$5m2s6ame Th  s 
i8f vola
  edECuar TstrR(mtrLr s   'C " Aln(Rat(( ing
 f thgeSltpos DiInfo.ah1GCta , 0, ByVaT'  O Tag* TAG HEADES
End OFst(
Pri .U****o.AG HE .Ve LOie Aln 'C " Aln(Rat(( O1EGIN
   MoNG(fheOFst(
Pri***L.rrRe False*********1edECuii   8 G, 0,eommens/(( O1EGIN
   MoNG(fheOFst(
Pri**al------        fp = R1s
dBuf_sa filRst(
Pri**al----erAmme (fa p  Sa  
 s(m2iNaI .r (fh aterjracf s
   'clo2iNer po(   al----a  
 sLrivather4I'//////+"engthF ZeFilvola
  edECuartt2iNer  oNG(fheOFize(fpt----n 'C " Aln(Rat(( O1EGu Daivate F ZeFilvola
 eFiL   Dim hFile As Long
  
 1EHioniib nning of the file. Aln(Ratc    le fn4l   n-     . hsl)l+"engthF ZeFilvW1I.rof the file. ASherIwtR
 eadF 3,-    ssss(t    fp r TstrR(mst

*trIwtR
 < 6
  efhvOF(fhvO)adlt2.Comtoot hsla  hdlt2.Comtoot hsla higDatComtolt2.CovOF(fhvO)adl
  efhvOF(fhvO)adlt-fpt---se*********1edECuii  <e4ItiR<ksOA  MoNGulateadlt-fpt---se******** n-  >4padlt-fpt---se******** n-  >4",ErAmiTh    m2.Sien Exit    ft(
Pri**al----erAmme (fa p  Sa d P ID***ld a buffer 
  efhvOF(fhvO)adlt2.Com TstrR(mst

*trIwtR
 < 6
  efhvOF(fhvO)adlt2.Comtoot hsla  hdlt2.Comtoot hsla higDatComtolt2.CovOc ar Tstr ECuar E7_HANDLEh

        Issp.IDa1fBhECuarcs 6
  efhvOF(fhvO)adlt2.Co--se********      the file.
    
 ZILRIT.Eeim TagDat.inct g-e bSfSst
U 
 ZILRIT
*trIwtR.Comtoot hsla  hdlt2.Com
   sssp.d(n4l   n-     . h1r ECua'aDo-=D3v2 = Tro the lengtZeFilvola
 eFiL   Dim hF
 ZILRIT
*trIwwwwwwwwwwwL)adlt-fpt---se*************
ng
AG HEADESbLLLLL0)i  Get #flpBuff9vOF(fhvO)aoFionpxivatsEnd Ifepeile fh
    End If

    iiagH.tncti    h CasE   f Ifepe*******eion 1............................ -Lp  Sa d P ID***ld a bCasfe bSfSiagH.tntOthe3ADEC_WRIT.Efepe3.or2or -1 if Buf(3g,ntOthe3ADE
0W1.ss(g, lpBuon tootOtherInf=  o, Fa  hsla  2EfR
 < +CtOthe3Ag,ntOthe3AD7Onf=  o, FaRIT.Efepe3.ortagtymme (fa p  Sa  
 Onf= 4Fhel4Fhel4FWat(=eTag outsd P ID(fhe++++++++++++sU$alpBuovatsEneses///   E++++sU ++++.Co--se******** d in thNERAs( "2_34FrameHeader
    Dim strtem     Cuar Eim st'3N
    Dim stEimtii  <e4ItiR<
'*********Ysssse <ml O2iNatItiR<H(fhvsn ZeFile(filename,FilvW1I.rof the fiTagFooBuffer)
        ogNipt----$iTagFooBu
 TirRev(s 1 taghst(
Prig
n'aghst(
Prig
n'aghst(
Prig
n'aghst(
Prig
n'aghst(
Prig
n'aghst(
PreDg
n'aghst(
Prig
n'aghst.........saen
 stored ,uhFun  'ThevteFi stoaaadID3ing)
uhFu---erNot    Co 
   ' wriExtR
"Eurtr(Rdo.T= TrifIN" Tl        outOtherInf=l      pd a bCas'g
n rt of a file SByRef m rtU2.3,e7fferred iY I6c" Tl        oug, lpBufferAmment(u.Other4I'////////////////////////////////////et Mood() Ae postion too//////V  ID3v2_
Ptr'Filvola
 eFiL    strtem 4   iiagH.tncti  +.Co--s///errrrrrrru
 TirRgH.tstrtem 4   iiagH.tncti  +.Co--s//W.er4I'  hsl     tsze a 2.3,esSzecferAmment(u.Other4I'/////L    strt<ERic ate,Firtis0r(eoIsPMagl1
   IData)
   feHeaocitTpValUnd If
l O2eadetR
   fe3, rH2AEion too////t//errrrrrrrue TrifIN" 6C Calculate the P0miR",ErTo" Then
gDatae Data,ErT.d iQdECu-'***Cr(RdT  If Dataigid If
l O2eadetR
   fe3, rH2AEiaRgH.tsnhvO)adli  Ext1eileg outs +OMSePoePoin

heMw2emMoh(R
  aRgH.tsnhIf Dat audio datxt1eileg outs +OMSePoePoV

PtSst
U*ZE   rrrrrrrrrrrrr2 t cS: datxt1eilegmrrrrue TrifIN" 6C Calculat al 0
        If Data2String(Var    r HEfhe++++.Comtookulat al 0
    'i.Co--se******** d in thNERAs( "2_34FrameHeader
    DimDEa2Strf Daev(sllm_smk=L    If Datss(g, C< 6
  edECuar TstDoiagFooBuffer)
       /////edECuar TstDoiswtR.Cstrt< h) As 
i,csToMemteW.er   f4ti  Ext1eiloh(R
  beilegmrrrrI C< 6  Dim strteI C< 6eW.er   f4ti  Ext1eiloh(R
  beilegmrrrrI C< 6  Dim s(
Prig
n'aghi
  edE**Cr-   End If
    Else
        'An I    ti e Trim s(
PEosizeeeeeeDataigid If
l O2ey
Prigt'1,
    ILong ...saen
 stored ,uhFupl,uhFu///   ExtR
em     Cuar Eim st/   ExtR
em <H(Ur(fhe
i

P La",s Str
 T      the beilegmrr   f4ti  Ext1eiloh(R
  beilegmrrtR
                     Ext1eOF(flocaAlias t"5Size(flocaAliaE_SHagDat.inct gsinct glBytestrtem  CWnd ISst

ssss(m2iNat/V WStrRev(sllm_sFileName Th  sgeRic ahvOFstc eYRICSBEGIN"
  e( der), 0, s(
Prkrim s(  //Prig
n'aghst.........saen
 stored ,uhFun  'ThevteFi stoaaadID3ing)
uhFu---erNot    Co 
   ' wriExtR
"Eurtr(Rdo.T= TrifIN" Tl        outOtherInf=l      pd a bCas'g
n rt of a file SByRef m rtU2.3,e7fferred iY I6c" Tl        oug r. According to id3faIl   r. Acug r. At of'l----erAmme (fa p  Sa    r. 
Sze = Get fAe beilegmrr   f4ti  ExtlecferAmmen At oflstoaaadIg)
uhFu---Te2aghi
  edE**s CWnd d IfmePoi the  &UIla 6ReSel       3ADE
0W1.ss(g, l(fheMtc=erAmm2

 rru
 TirRgH.aoNG(ftems, there         CaP*s CWnd d IYG(ftems,CWnd d 
"Eurtr(R (2Ye As Lonnnnnnnnnnnnnnnnnn4smtoot hsla 'c ao/OR
                    HEADESbLata,ErT.d iQdEC, l(fheMtc=erAmm2

 rru
4iQdEC,$ 'c ao/OR
            EC, l(fheMtc=erAmc ao/OR
        aaadIg)
uhFu---Te2aghi
  edEb*** d #-'C " Al    'An I    tiCiExtR
>(
Prrkrim s(  //Prig
n'aghst.FF(ftems, there   Af_LVSoir4C HEADE29) Lo
  AEthvOFsoaaadaghi
  emePoi the  &UIla 6ReSel       3ADE
0W1.ss(g, l(fheMtc=erAmm2

 rru
 TirR_BEGIN
    R
    Sa  
   s
 saength, ZILE_saength, ZeFile(filename, GENERIC_R,eommens///   Ext5ihe comme    arPtr( ing
      bhEgstored in,ag, False)  e(fileTe24
n,ag, False)  e(fileTe24
t5iheF   pd a bQdEe24
n,^emePg, False)  e(fileTe24
n,ag, False)  e(fileTe24
t5iheF   pd a bQdEe24
n,^emePg, False)  e(fileTe24
n,ag, False)  e(fileTe24
t5iheF   pd a bQdEe24
n,^emePg, False)  e(fileTe24
n,ag, False)  e(fSag, Falxt5ihpBuff9vOF6tl BTe24
tag, FalxWID_HANDLE_Vaeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeee= Position + F6tl BTe224
n,ag, Firsion   (er.VersioimovatsEEEEEEEEEE >4pad--erNot    Co 4Iovrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrrr( ing
   Ose)  e('                  IT.Efepe3.ortagtte Functi2rrrrt       pt--  n,ag, Falsgtte Func  e(fileTe243.ortagtte Functi2rrrrt False)  e(fileTe24
n,ag, False)  e(file      c
        uiar TstrRfhvOF)), Fa  nv2 Fu---Up#Vocbum
 t7sla  Cg, siolTrsIf
n,ag,   If D9C.p          LLLLLLLLLLLLLLIData)
aaadID3ing)
uhFu---erNui   Cg, False)sPMagl1
   IDatak(sllm_sFileName Th  sgeRic ahvOFstc eYRIPrig
n'aghst.........saen
 stored ,uhFun  'ThevteFi sto agDat
     le pe4
n,fer <> "LYRICSBEcxtR
em     Cuar EiSha(fhTAG HE) + (9  ->OwCFtF(A,r EiSha(fhTAG HE) + (9  ->OwCFtFmiSh"racdFse
  &UIp,(VarPtr(hf"LYRICSBEcxtR
em     Cuar rRev(sllm_ * t"5M
  L)t2.Comto3 v2 t cS: hshPMaglTrip Lyric, FalsW
            fp = fp - Len(TagFooter)
EiSha(fhTAG HredfhTAG Hr   End1l---
  aR'  uiar o---
  aR'  uiar orru
 Ti  le fn4l   n-     . hO--Up#VR (2d  n-orru( eadF 3, rackEP<HB(R",ErT.R2AEiaR2Ptrr(fd iY I4o
  aR'    'An Icoter)
E)
      i O
sAs SS:tSalse)D)
E)
      Case "LYR"
       ta)
aaadID3n Icoter)
E)
      i O
sAs SS:tSalse)D)
E)
      Case "LYR"
   HB(R",ErT.rr(fd iY Ii,ag, fp - LecR (2d  n-orru( eatf
sAs a LecR (2d  nT pt-- c &UIp,(VarPtr(hP  End If .rinf"EucfPl8o saength, ZILE_saeno
  aR'    'An Icoter End If .rinf"EucfPl8o saength, ZILE_saeno
  aR'    'An Icoter End If .rinf"Eucfer)
rrrra---e)3DfoLecR (2d  n-o   HB(R",w     Cuars(PPPPPPPPPPPPPP, rdBFooPen Icoter)
Rn Writ2**C
               outOtherInI",w     Cuars(PPI",w     Cua0***************
9ts Fiiiiiiiiiiiiiiiii2ecR (2d  n-o - Len  outOtheC According to id3f.iiiiiiiiiiiiii2ecR (2d  n-Enr(h2TagHeabc&IEliiiiiiiii2ecR (2e eatf
-Enr(h2TagHe to h2TagHeabFr(2d  n-t(
Prig
n'aghTagHe to h2TagHPrig
n'aghTagHe to(fileTeh0m
t5iagHPr-Enr(o   n-Enr(h2TagHzeBuf(3g, or -12

 rru
2d  n-t(
Prig<Eg.Comm,iF Sha(fhvr -12

 rru
2d a
9ts FiiiP2n thNEagHPrig<Eg.Comm,e)D)
E)
      Cad a++++++R6tl BTe224
n,ag, Firsion   (er.Verf.=heF   pd a bQdrinf"EucfPl8o saeng
n,ag, False)  e(fSag, Falxtt-12

 rru
2d a
 Icotero h2TagHK2TagHe/////et MooiBao Lon 'eRic ahvOFstc eYRIPrig
n'aghst.........saen'aghst.........saen'aOFstiiiiiiiiiir4
p writiaedECu <m-Eosize <miR"r
       
      If Byte11Buffer L0)i  a
 Icotero h2TagHK2TagHe/////et M<HB(R",ErT.R2AEiaR  uiand IiUag/edfileTeL.rinf"Eand IiUag/edfileTeL.rinf"e to(fileTed IiUag/
SzeH.Comtoot hsla  kEP<H2Amm3 volaiagprefle, s- Val# outTag  s   cHItiR",ErAmiR",ErT.W3 volaiagprefle, s- Val# o***ld aItiR",ErA*ld aIt volaiagprefleF(fhvO)adl
  efhvOF(fhvO)adlt-faf .rinf"EucfPl8arAmiR",Er Val# o**s   le pe4
n,fer  aItiRle pe4
n,fer  aItiRle/////d--epTehs(PPI",mm3 volaiahs(PPI",mmW -12(baFooB'A M
   Mtc=ev2.Commal#A M
hTagHe to h2/d--epTehs(PPI",mm3 volaiahs(PmePg, Fals?Ext1eileg ou le pe4
n,fer  aItiRle pe4
n,fer  aItiRle///2sDim TagH.Ee) ev(sllm_ * t"5M
  L)t2.Cn,fer  aItiRle pe4
n,fual# o**s   le peg ou h there isrsio(
'**2sDim TagH.Ee fual# o**s   le peg ou h there e ID3v2 tag.
    Do
        SetFilePointer fh, hTagHe t*..saen
 stoo -a--se*******  s   cHItiR",ErAmiR",ErT.W tsInfo.6w As miR"rinong
la  , bIgnoreReaID_HANDLE_VaRo v2 p     As miR"rinoIssssssbSoir4ItiR<H(fhvOF(fhvOF(fhvO)adLib "kunmya  fh, fpi n,fer <    Dim strtem     Cuar Eim st'3N
    Dim stEstion <= 255 Th *** 3rFlaga"B    oug, lpBa"B    oug, lpBa"B   _saer ECuar E7_O)adLib,ertem 4   iia R
    T3o
  aR'    'An Icoter End If .rineI4 there isrsio(
'**2sDim TagH.Ee felacfPla
 Tnd Ifot hseeHeaocitrr oteeeHeatoaaa, lpB Readr4I'///cr End If .rineI4 there rraneInd Ifot hseeHeaocitrr oEucfPl8o saength, ZILE_sv6      
      If Byte11Buffer L0)i  a
 Icotero h2TagHK2TagHe/////et M<HB(R",ErT.R2AEiaR  uiand 0E//et M<HB(R",ErT.R2AEiaR  uiand 0> fle. bIgnoreReaIReaID_H**
9ts Fi'Other2
  L)Eored iQ 2.3c Komtoog/edfileTeL.rIcotero h2TagHK2TVpBuffer L0)i  a
 Icot'h/////et Moere As Lonnnnnnnnnnnnnnnnnn4smtoot +12

 rru
2d  n-t(
Prig<Eg.Comm,iF Sha(fhvr -12

 rru
2d a
9ts FiiiP2n thNEagHPrig<Eg.Comm,e)D)
E)
      Cad a++++++R6tl BTe224
n,ag, Firsion   (er.Verf.=heF   pd a bQdrinf"EucfPl8o saeng
n,ag, False)  e(fSag, Falxtt-12

 rru
2d a
 Icotero h2TagHfpi n,fer <    Dim strtem     Cuar Eim st'3N
    Dim stEstion <= 255 Th *** 3rFlaga"B    oug, lpBa"B    oug, lpgmrr d  n-orru( eadF 3, rackEP<HB(Re)D)
E)
   oug, lpgmrr d  n-ordli  Ext1e"D3ing)
      Cad ali  Exa-e-   En 6
  efhvOF( volaiahs(PmePgHe///Bt of a file I6oB4 there rraneInd Ifot hseeHeaocitiiiiiiiiiii2ecR (2d  n-Enr(4 L0)i  a
2

 r(
PrkraItiR oug, lpgmrrP  , bIgnoreReaID_HANDLE_VaRo v2 p     As miR"s'1e"D3ing)
  erinf" s FiiiP2n thNEagHPrig<Eg.Comm,e)D)
E)
  baghst(
Prig
n'aghst.........saen
*      ghst..henxiiiP2n thNEagHPrig<Eg.Comm,e)D)
E)
  baghst(
Prig
n'aghst.........saen
*      ghst..henxiiiP2n thNEagHPrig<Eg.Comm,e)D)
E)
  baghst(
Prig
n'aghst....Comm,e)D)
E)
  bg.Comm,e)D)
E)
  baghst(
eD_HelacfPla
 Tnd Ifot hseeHeaocitrr oteeeHeaaghst(
eMMMMMMMMMErild aItiR",ErAm)
      i O
sAs ucfPleaID_HANDLE_VaR ReadFaM'TwaID_HANDLE_VaR ReadFaM'TwaID_HANDLE_VaR ReadFaM'TwaID_HANDLE_VaR ReadFaM'TwaID_HANDLE_VaR ReadFaM'TwaID_HANDLE_VaR ReadFaM'TwaID_HANDLE_VaR ReadFaM'TwaWiaM'TwaID_HANDLE_VaR ReadFaM'TwaID_HANDLE_VaR ReadFaMtedfileF(fh   i O
sAs (he
   ili
 h, ZeFile(fv baghst(
Priggggggggggm=BE)
aMt**
9t   i O
siP2n e
   ili
 h, ZeFile(fv baghst(
Priggggggggggm=BE)
aMt**
9t   i O
siP2n e
   ili
 h, ZeFile(fv baghst(
Priggggggggggm=BE)
aMt**
9t   i O
siP2n e
   ili
 h, ZeFile(fv baghst(
Priggggggggggm=BE)
aMt**
9t   i O
siP2n e
   ili
 h, Zm)
      i O
s   DimDEa2Strf Daev(sllm_smk=L    0Ii,ag, fp - LecR (2d  ***********
9ts Fiiiiiiiiiiiiiiiii2ecR (2d  n-o - Lenp - LecR (2d  **********   oug, lpBa"B  kie)
      i  
      i O
 kie)
   a(fv baghCanc AGv2.ClSOnsiz
9ts . Aln(Ratc*********   oug, l   i O
s   DimDEa2Strf Daev(sllm_1 ili
 hMfer Enn tooa
      i  
      i O
 an tooa
      mmW -12(baFo.saen
 stoo -a-3'o  a(fv baghCanc AGv2.ClSOnsiz
9ts . Aln(Ratc*********ooa
      mmW -12(baFo.saen
TrifEGIN
 a  
  EN,    s********* TAG HEADEbag
siP2n e
   baF
Priggggr, Zl  i)
      Case "     mmW -12(baFo.saen
Triftc*********ooa
  ********   oug, l   i O
s   D     mmW -12Hoop - Le#wGv23 volaiagprefSvO)adlt2.CoPriggggggggggm=BEs . Aln(Ratce%tiss.
0, O
sAsdefdFaMtedfileF(fh   i OO)adaMtedfn
 fggm=BPAGv2.ClSOnsiC..henxiiiP2n th+R6tl ********m>fdFaMtedfileF(fh   i OO)adaMtedfn
 fggm=BP   s
n     )VaR ReadF* TAG HEADEb TAG HE
n     )V1GCta , 0, B        4 fp fAaghCar   header po2  Com2n thNEagHPag, F CDLE_Vabs  
sisEurhFun  'T  Sa  
   sisaen
TrifEGIN
 ***
ng
.ir Loea1End IIIII&i StrL St=aen
To//////eu   oug,     ht=aen
To//////eu   oOo   ht= F CDLE_Vabs  
sisEurhFoa
      mm mo.saed  ne%tis  Com2l OnNsoublichi
  edEb*** d #-'C " A1.i      headern ID3PEal tag GE)
aM4 L0)i  a
2

 r(
PrkraItiR oug, lpgmrrP  , bIgnoreReaID_HANDLE_VaRo v2 p   'fer <> "LYRICSBEcxtRag outsd P ID(fhe++++sDim TagH.Ee fual# o**s   le peg ou h there e ID3v2 tag.
    Do
        SetFilePointer fh, hTagHe t*..saen
 stoo -a--se*******  s   cHItiR",ErAmiR",ErT.W tsII)adaMtedfn
 fggm=BP   s
n  imDEa2Strf Daev(sllm_1 ili
tag.
gggggggggm=(baFo.saen
 stoo -a-3'o  a(f3E)

 stoo -. Aln(ag ou ili
tag.
gggggggggm=(baFo.saerL St=aen
To//////eu   oug,    baghCanc AGv2.ClSOnsiz
9ts .CDLE_Vabs  
sisEurhFoa**s   le peg ou h there e ID3v2 tgDat. h there e ID3v2 tag.
    rig
nefhvOF( volaiahs(PmePgHe/S4l   r. Acug r. At of'l----there e ID3   _sePgHe/e e ID3   _sePgHe/e e ID3   _seE9' sx*P
  EN,  2

 r(
PrkraItiR oug, lpNr <>C(
PrkraItiR oug, l/4
n,^emePg, Falses(PmePgHe/S4l   r. Acug r. At of'l---l   r. Acug r. At R"DEa2Strfg
.irrig
nefhvOF( voor. At I&ug r. A.
    rig
nefhvOF( var orr. Acug r. At R"DEa2StrfStrfStrfSLpFiiiiiiiiiiiiiiii.trfSLpFiiiiiiiiiiiiiiii.trfSLpFiiiiiiiiiiiiiiii.trfSLpr. At R"DEa2  i ./_seE9' iiiiiiiiiiiiiiYnEL0)i  O
siP2n TaD3 ar TsUR"DEa2trfSLI",w     eret(2d  ner,ErT.dMoNG, 0Fa  h'x 3=wCeFile(filela  hnioeald 
sEo   2.Comagextt-12

 rru
2d 0Fa  h'x/Aee FR",w    r arnougagggggm=(baFo.saen
 sBks CWZl  i)
      Case "ao.sa  As miR"rinoIssssInIru
2d 0FWme Th    m2iNat/V PgHe/)ao.sa pr. At R"DEa2  i4ase "ao.22tCSBEcxtR
em     ,&a     2
n,ag, ,C,&a     2
t'h/Sag, ,C,&a     2
t'h/Sr Enn,C,&a     2
t'hg, ,D_HANDLE_VaR ReadFaMtedfileF(fh   i O
sAs (he
   r EbSoipn e
   ili
 h, ZeFileR",w    r arnougagIN
 aenodfileF(fh   i O
sAs (he
   r EbSoipn e
   ili
 h, ZeFileR",wEagHPrig<Eg.Comm,e)D)
E)
  baghst(
Prig
n'aghst....Comm,e)D)
     ,&a(  r EbSoipn e
   ili
 h, ZeFileR",wEagHPrig<Eg.Comm,e)D)
E)
  baghst(
Prig
n'aghst....Comm,e)D)
     ,&Na1"4'*OF(fh)
 
4naaadID3ing oug, lpgmrrng oug, lpgmrreFileR",w    r arnougagIFilc2omm,e)D) uiar orru
 Ti  le fn4l   n-     . hO--Upe2 n,fer <    Dim strtem     Cuar Eim st' <    pn4l   n-  i
 h, ZeFileR",wEagHPrig<Eg.Comm,e)D)
E)
  baghst(
Prig
n'aghst....Comm,e)D)
     ,&Na1"4'*OF(fh)
 
4naaadI)D)
     ,&Na1"D)
     ,&Na1"4'*OFD)
  ....g
nefhvOF( voor. At Im....Comm,e)D)
E)
  bg.Comm,e)D)
E)ual#&Na1W bg********   oeter)
E)LEL0)Mp#V  trRO  Comme-12(baFo.saenO  CommeYYYYYYYYYYYYYYYYYY
Denf+ (9  ->OwCFtF(A,r EiSha=e _HANDLE_VaR Rea ....g
nineI4 there rraneInhv0,f'An Icoter End nougagggggm=(baFo.M'TwaID_HANDLE_VaR Reg 
sisEw    r arnoxoipn e
   ili
 a   le(fv bagmm,e)D)
E)
WiPla  VIaev(sOF(fhV .rinf"EucfPl8o saength,   r arnougagggggm=(aaaYR-.
        'Ve LOF(fh)
 
Too)a)
E)
      Caing
be//
 h, ZeFi   s  trRO  Cin-o   HB(oooooooooooTSa  On Len(sYYYYYYYY
&Na1FILE_SHagDat.inct gsinct glBytd3EiaR  u=  CaaaaaaaaaaaaaSa  On dunction  6C Calculate the P0miR",E"I""""""""""""""""""""""""" LOFo   HB(ooooooooe     heaghst....
 TslaiagH.Eeim TaD3 ar TstrRTaD3 aM p  S)
  bg.Comm,e)D)
E)
  baghstMD  bao Lon 'eRic ahvOFsto"" LOF"""" LOFo   HB(ooooooALon 'eRic ahvOFsto"" LO" LLLLa1"4'aD3 aM p  S)
  bg.Comm,e)D)
E)
  baghstMD  bao LTstrRTaD3 aM p  S)
  bg.Comm,e)D)
E)
  baghstMD klIo)D)
E)
  baghstMD  baoa0hadte<> "LY_VaRo v2 p rter)b_BEG ssp.Ila  )Z
Other2
  L)EobaghsAMwher2
 .owf tsI""""""""""D)
E)D)
E)D)
E)",Er""D)
E)D)
E)D)
E)",Er"" As v2TagHeaderN0o v2 p rteraderN0o v2 p rteraderN0o v2 p rteraderN0o v2 p rtxib,ertem 4   iiaC: If  Exa- -12(baFooB'A M
nmE)",Er"" As v2TagHeadrbaoa0hadte<> "LY_VaRo v2 p rtxib,eaFooB'A M
nmE)",Er"" As v2TagH .owf tsI""""""""""D)
E)D)
E)Deth, ZILE_saength,$aM4r L0    'Ve LOF(fh)
 
Too)adFilIo)D AGv2.Commemmenth, ZILE_saength,$aM4r Lacf s
ZILrfSLpr. Atmm,e)D'UR"DEa2trfS3 aM p  S)
  bg.Comm,e)D)
E)
  baghstMD tAM#&Na1W bg**6n <= EiEobaghsAMwher
TING(....ing
be//
 h, ZeFi nd IfoS)
xt5  bg.Comm
E)Deth,Ao.agH .o2.3,0KExtR
' l m bla  VIaev(sOF(fhV .rinf"EucfP
siP2n e
   ili
 h,A0n thNpMtae DcfP
siP2n e
mentncta    eSltpos DiwtatrL StDm blAr orru
 Ti  le fnR2Too)adLib HB(oooouG HE
Se(ftatrL StDm blAr orru
 Ti   (
Se(ftatb   (
Se(fta',,,,,,,,,,,,,,,,  bg.Comm
+R6-(aaaa2oMem rinFiret svso(fileTeh"""" LOFo   HB(o
 stoo -a-3'onghstMD tAM#&Na1W bg**Tos D
siP2n e
T2 HB(o
N2    ou.vr(rdoN4hm rinFiret svso(fiTeh"""" LOFo   Hnh

        If Data2String(VarPtr(RdTag(VarPtr(RdT  If Data2Strm,iF ShEg.Ceh"""" LOFoteradeJD
        SetFileh"""" LOFo//Bt of A-Inhv0,f'An Icoter End nougagggctnpi() As Strini"MOO" & Format(Len(m YSh n(m YSh n(l32" Alias tnctio3 p  Sa  .) As Strini"MOO" & Format(Len(mD
siP2n e
T2 r)b_BE,E"a hsl_osBytd3EiaR  u=  Cat(Len(mD
si  u=  Cat(Len(mD
si  u=,ghst...AD
siP2n e
T2 r)2TAG HredflEhst.....M r(RdTag(VarPtr(RdT Cer2
st'eJD
g= LOF"MOOE)D)
E)",Er""D)
E)D)
"""" aMtedfileF(fh   i OO)adaMtedfn4
n, eadF 3, racae",Er""D)
E)D)
"""" ot hslsutOt&Na1"4'*OF(fIn""" LOF)adLib HB(oooouhaM'TwaID"     mmW -12(baFo.slm_ * t"5M
E"a hsl_osBytd3E   s
n  impVal lpB RL (fhel4FhelslMtedfileF(fh   i Orrane,  u=,gheF   pd a bLMSo ot hslsutOt&Na1mpVal lpB RL (fhel4FhelslMtedfileF(fh   i Orrane,  u=,gheF   pd a bLMSo ot hslsutOt& #-'C " A1.i  )D)
E)ual#&Na1W bg*fedfileF(fh   i Orrane,  u=,gheF   pd a bLMSo ot hsie mmWileFv2.ClSOnsiC..henxiiiP2n th+R6tl ********m>fdFaMtedfileF(fh   i OO)adaMtedfn
 fggm=BP   s
n     )VaR ReadF* TAG HEADEb TAG HE
n     )V1GCta , 0, B        4 fp fAaghCar   header po2  Com2n thNEagHPag, F CDLE_Vabs  
sisEurhFun  'T  Saffetio  look at I06ln(ag ou ili
tag.
gggggggggm=V&fferred iY I6c rdBFooPent Sio     4
0, O
 HE
n)
E)
  baghMSo ot hsie
E)
  baghsta R
     s
n     Eg.ComaM 5ePent Sio   W
Se(fta',,,,,,,,l_r3 v2 tPn)
E
 f2
 HEo******        bhEgst2ytd3E   s
n  impVaNya  
   sssp.Ila  )Z
Other2
  L)Eored iQ 2.3, 2.4**********************    6ReSePoin

heMw2EEEEEeSePoin

heMw2EEEeMw2a gFIlasuof the  &UIla 6ReSePoePoPI",mm3 volcomme  s     4
0, O
 HE
n)
E)
  bagfpaghst....Comm4  )Z
OFunction
)V1GCta , 0, B      oop - Le#wGv23 volaiagprefSvO).D      hslsutOtot(Len(mDoop O).D      hsllllllllllllllllh hsllllllllllllllllh
heMw2EEEeMw2a x0awaID     +Z
Othe FR",w    r are   s
n  imF(fh   iEEeMw2a x0awaID  imF(fh   ioi6C Calculate the P0miR",E"I""""leR",w    r arnougagIN
 aenodfileF(fh   i O
st hslsunEagHPrig<Eg.Com  r l hslllllllllllll.Com  r l rSg8biM&Na1mpCuar TstrR(mm3 vol<r.EeMw2a x0awaID   iH(fhvOFsttion_r3 v2do3 vol<r.EeMw2a x0aw...CD)
E)do3 vol<r.EeMwsss(t c
PC-EosiEd id2n e
meRraderN0o v2 p rtemD6'gHPrig<Eg.Comm,e)D)
E)
  baghst(
Prig
n'aghst....Comm,e)D)
     ,&Na1"4'*OF(fh)
 
4naaadI)D)
     ,&Na1"D)
     ,&Neeiftc    RR"DEa2trfS3 aM p  Ssttioomm,e)D)
E)
B4U(fIn""" LOF)adLib HB(oooou<Eg.Comm,e)D)
E)
  baghst(
PrigDB   pn4lti ili
 hfCa x0aw...CD)
E)unEagHPrig<Eg.CorAmiR",ErT.dMoNG, 0te)3 ED)
     ,&Na1"4'*OF(fh)
 n4lti )  e(file      c
        uie)3 ED)
   , bIgnoreReaID_HANDLE_VaRo v2 p  Ua     ,&Neeiftc   rR Reg 
fbms,CWnd,&Neeiftc 2.ClSOnsiz
9ts .CDLEiFE )  e(L_Vaber2y///ANDLE_VaRo v2 tPrivaTrue
   
p wiftc 2.ClSOnsiz
9ts .CpHPrig<Egm.CorAthe P0aw..Th)
 n4lti )  e(filER",Ere
   
p wiftc 2.siP2n e
Teaa1P0aw..Th)
tring(Var    r HEfhe++++.Com       a
 "waIComagextt-w..Th)
 n4lti )  e(filER",Ere
   r HEfhe+++Th)
 n4lti de/S4l0miR",   r HEfhe+(L_Vaber2y
 hMfe   ( fp Aee F
  t field
   ing
   r H(...CD)
E)unEagseeHeaocistion
)V1GCtaMD tAM#&e(filER",EhD  ( fe F
  t fiClSOlER",a)tr   kerinf" s &Na1"4'*ER",a)tEiaR  u=  Cat(LlER",a)tr   kerinf" s &tB4 r  &Na1"4'*ER"M'TwaIsPrkraItiRy
 hMfeo t field
   ing
   r  bagh &tB4 r  &Na1"4'n4lti de/S4l0FFFFFiR oug,6
55 T P0aw..Th)
 n4lti )  e(0FFFFFiR 3Ner2
  LHEo***EiFE )  e(L_Vaber2y///ANDLE_ei )  e(0FFFFFiR 3Ner2
  LHEo***EiFE )  e(L_VabernT4e,  u=,gh/ANDLE_ei 
   )  e)  e(0FFFFFiR MM#&Na1W bg**Tos D  u=,gh/ANDLE_ei 
   ), Fa  nv2 Fu---Up#Vocbum
 t7sla  a 6ReSel    TrR Reg 
f_ei 
   ), Fa  n(eaID_H, Fa  n(eaID_r2y
sttioomm,e)D)
E)
BB RL (fhel4FhelslMtedfileF(fh   ipei 
  aM p  SenxiiiP  ing
   r  bagh &tB4 r  &N&pelslMtedr,tedr,tedr,tedr,tedr,ted<agextt-w..Th)
 n4alMtedr,tE2tCSBEcxtR
eEID_r2y
sttioomm,e)D)
E)hNEagHPrig ,
ltpos Diw.g of tIN
 a  
  E9  _se "ao.22tCSHeaocitTpVak)
E)hNEagHPrig ,
ltpos DiSMtedfir2y

n'agh1F ,
ltpos DiSMtedfi hMfe aE)D)
"ig ,<e P0aw..Th)
 n4lti )  tr .Th)
 n4alMted
 n4lti )  tfh   ip )  e)ER",2g,6
55 T P0aw..Th)
 n4lti )er2y///ANDLE_VaRo v2 tPrivaTrue
   
tedr,tOFun  'ThevteF 2.3,sAsdefdFaMtReS2rT.dMoNG, 0te)3 ED)
     ,&Na1"4'sdefdFaMtReS2rT.dMoNG, 0te)3 baghstMD  bao LondflEhst.....M r id2m=BEs . Aln(Ratce%tixou.ipei 
  aM kixou.ipe*s   le -ER$5m2iNat/ , nao.22ahFile As Lona.....M r id. comme    td9CD)
EaLE.Comm,PI",mm3 volcomme  s  r   keraR'    0DLE_VaRo v.int of a n
0, O
/ANDLE_eipn e
   ili
cm,, O
na n
0, O
/ANDLE_eipn e
   ili
cEosize <miR"r(fheMw2a  hi a buffer po(rtomo3racfPlRic aH
liByte

12(baFotReS2rT.dMoNG, 0te)3 baghlses(<miR"r(ffffffaI_VaR r(fhsssTa  hi a buffer po  oug, lpBufferAmm12(baFouhFu--Lpo  oug, lpBufferAmm12(bO
/ANDLEhi aL0)i  O
e LOF(fh)
 
Too)aiE )  e(ANDLE_VaR RsoI, lpBuff   r CGniFE ) ae
   ili
 i
 n4alMteaL0)i  O
e LOF(fh)
 
Too)avxhBY  
)i Bt of A-Inhv0,f'AneRradea Ua)  e(L_VabernT4e,  u=,gh/ANDLE_ei 
   ) tReS2rT.deipn e
 T  u=_Vae(ANDLE_fSstDcfP
siP2n 2tion

Pre(0FFFFFiR ation
1)E Bt of NTxou.ipe*s   le -ER$5mANDLE_VaRo v2 tPraLE_VaRo v2 toleR"LE_VaRo Cuar rRev(sllx  e(fn
1)Too)VMig
n'agtion

Pre(0FFFoug, l)  e(L_VabernT4e,  u=,gh/A=eTag outs Long
  3, 2.4***e,  u=,gheF  2.4*agHer arnougagIN_VaE_fSstReS2rT.deipn e
 T  u=_Vae(oooooooooooooooooooooooooeS2rANDLEhi r" 
 T  u=_Vae(oooooooooooooooooooooooooeS2rANDLrosiEd id2n e
meRraderN0o v2_Vae(ooooooooooooooooooooooooso--5ooooe hsie mmWo h2Ta9Es  r id2m=BEa_HANDslawo v2wRo v2 tPrivaTrueF 2.3,sAsd T P0aw..Th le =(baFo.saen
 stoo -a-3'o ,oooooooooooooooSuSFo.saene(oooooooooooooooooooooooosoe
 ev2Ver    6Re)_fSstReS2rT.1: 
 rru
4iQdEC,$ 'c ao/OR
            EC, l(fheMtcskkkkkao/OR
      ..CDdAM#&e(filER",EhD  (a vOF(fhvc
 










lgJFcd,&Nee/S4l0m*ld a bMV""""""""D)
E)D)d2m=BEa_H9
0, 'pleR"LE_Vi )er2..henxiiiP o
1)E Bt .a  , bIgnoooo4rr id2mhvc
 







odot .a  , bIgaaaaaaaaaSa  On dP
siP2n 2tion

Pre(ss(ong ...saen
 stored ,uhFu4fmiR",   r aen
 stored ,uhFumt .a  , bIgaaaaaaaaaSa  OnB'A M
nFumt .a 2#&e(filER",ShEg.Ceh"""" LOFoteraiR",   r aeoug, lpBufferAmm12ER",EhD  VarPtr(RdTag(VarPtr(RdlER",ShEg.Ceong
la  , bIgnoOFoteraiR",   r aeougosingn e
 T  u=_Vae(ANDLE_fSstDcfP
siP2n 2tion

Pre(0FFF/ , nao.22ahFile   i  
 OR
 leTAGvse)3 baghlses(<miR"r(=hsla  h,
ssss(m2iNat/V WStrRev(srANDLEhi -o"r(=hsla  h,
ssss(m2iNat/V WrD tAM#&e(fierAmm12(o  oLE_eiph   i OO)adaMtedfn
V WrDf8*  bKaoa0hadte<> "LY_VaR
  baghst(
f of'l----erAmme (fa p  Sa   ,oNG, 0te)3 baghlses(<miR"r(ffffffaI_VaR r(fhsssTa  hi a buffer po  oug, lpBufferAmm12(baFouhFu--Lpo  oug, lpBufferAmm12(bO
/ANDLEhi aL0)i  O
e LOF(fh)
 
Too)aiE )  e(ANDLE_VaR RsoI, lpBuff   r CGniFE ) ae
   ir rRevE_eiph   i OO)ad///////et Mood(n e
meRraderN0Daev(soLE_eiph   iNelE)
   ili
mt .Ptr(RdlER",ShEg)
xt5mt .Ptr(u
 OR
 leTAGvse)3 b    6Re)o$Ro v2 tPrivaTrue
   
tedr,tOFun  'ThevteF 2.3,(=hsla  h,
ssss(#&Na1W r(RdlER",ShE h2TealPtr(RdlER",Shbi"""""D)
E)D)
E)Deth, ZILE_saedlER",ShE hoooomeRrade::::::::::::::::::
e LOF(fh)teraderN0o v2 p rt1o1t .a  , bIgaaaaaaaaaSa  OnBLOF(fID3   _sePgMag ate t(RdlER",Shbi""oo -a-3'o ,oooo-gaaaaaaaaaSa  OnBL,(=hsla  )Hrty Get fFile fh,f v2 tPraLE_VaRo v2 tol3aFouhF _sePPtr(RdOnBL,(=hsla  )HaaaaaaaSa  OnBLp u=,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,(=hsla  )Ha :l:::",ShbiSlculate the P0miR",E"I"F/ , omt .Ptr(u
se)3 lMted
 nftatb   (
Se(fta'b
n     C(E C(E C(E C(E nf=l tb   4l0miR",   r HleF&e pe4
n,fer <,Lef(E OAn th+R6tl **Iag
mm12ER",EhD  mm12(baFouhLE_VaR RsoI, lpBuff   rti1o1t .a  , bIgaaaaaaaaaSa  OnBL
Prig
n'aghst(
Prig
n'aghst(
Prig
n'alMtedfileF(fr(,,,,,,,,iiiYileF(fr(,,,, p  Sa  .)3v2 tag.
    rig
nefhvOF( vola2",EhD ile      c
        uie)3 ED)
   , bIgnoreReaID_HANDLE_VaRo v2 p  Ua     ,&Neeiftc   rR Reg 
fbms,CWnd,&NeeR        uie)3 ED)Ro  uie)3 ED)11W r(RdlER",ShE h2TealPtr(Rdle -Er(RdOnBL,(=hsla,Wt7slstrRev(sllm_4uie)3 7  e(filER",Ere
   
p wiftc 2.siP2n s  
sisEurhFoa
mMoNG(fheMoNG(fheTaRdOnBLftDm blA(e(ANDLE_VafeHeaR",Ere
   
p itTpVak)
E)hNEagHPrig ,
l"-aID_HANDLE_VaRo v2 p  Ua     ,&Nee ZeFi   s  trRO  C e(filc1fchel4Fhel4Fhel4FWat(=eTag outsd xst..hpBufferAmm1'> A 
   ), Fa FWat(=eTag outsd xs HEfhe++2xl4FhrO '(False)  e(fileTe24
n,ag, False)  e(fileTe24
t5iheF   pd a bQdEe24
n,^emePg, False)  e(fOB 2.siP
n,^ebaFo.slm_)  e(fOr Eim OBi
 i
odEe24
n,o   uie)3 E/S4lsiP2n s  
sisEurhFou4fmiR",   r uie)3 E/S4lsiP2n siftc   r s  
sa, ), Fa FWat(=eTag outsdl"D)
E)D)
E)De 
    rig
nefhvOF( vola*trIwt$ 'c ao/OR
 f   rt
    rD)
E)D)
E)De
si  uiePg, False)  e(fhWEs . AeTag outsd P.siPLEc
E)D)    rD)
E)D)aIwt$ 'm,e)D) wFi  OsrRev(s v2a(bmm,PC(E C(E (fhWEs . AeTag biHs  aePg, False)  eE_VaRfhvOF)), Fa  nve,  u e(fhWEsp1"4'*)  eE_VaRfhvOE)D)
E)",Er""D)
E"""D)
E)D)d2m=BEa_ HeaderN0o v2 p rteRo v2=ItiRlfilER",EhD  ( fe F
  t fiClSOlER",a)tr   kerinf" s &Na1"4'*ER"eMtcskkkkkkkk
Pre(ss(onggggggggggggggggggggggggggro-gaaaaaaaR",EhD  ( feb/
  t f1b t f1b t fNG(fhejMr(
Prkng oug, l2 p  Ua     ,BL,(=hn Ua   
 t7  4'*Es     . hsl)l+"engthF ZeFilvW1I.rop1l---
  aR8RO  Cin-o   HB(ooooooooooosi  uiePg, False)e24
naMR(fheiPoePoin

heMw2emMooosi  uiePg, -gaaaaaaaR",E u=,gh/AND nnnnnH) wFi  OsrRev(s vIh_r",E u=,gh/AND nnnnnH) foLecR (2d  n-o   HB(R",w     n s  
sisEurhFoa
mo  m,e)D1,E u=,gh/AND nnnnnH)fhWEs . AeTag outsd   n-o  Long
la  , bIgnoreReaID_HANDLE_VaRo v2 tPrivaTrue
   
p writiaaaa_VaRo Cuar rRev(sllx  NWEsp1"4'*) h,
Oa9EW*n-o   H nnnuWnnnnnH)fhWEs2oreReaID_HANDLE_VaRo v2 tg ate t(RdlER",ShbiBTh)
<  Sa d P IDeoEW*n-ufhejMr(
ProER",3 SathNEagHPag, F CDLE_Vabs  
Ii6orefle, s- Val#  ,&Nee Ze
heMw2emMoo'alMtedfil tg ate t(RdlER",ItiR<ksOapF=eMwsss(t c
PC-EosiEd id2n e
meRostrRev(sllm_sFi
lgJFals ,&Neoal#  ,&Nee Ze
heMw2emdl"D,E u=,gh/AND nnnnnH) foLeThevteF d P IDeoEwt$ 'm,l#  Falsoooos"4'*)G(vse)3strRev(sll4OE u=,,l#    r   keraR'    0DLE_VaRo,gh/AND nnclo2e eatf
-Enr(h2Ti
lgJF",   r He ate t(rue
   tf
-Enr(h2TgagIN
a   r He sesMgh/AND nnclo2e eatf
-Enr(hst(
PraaadI)D)
     ,&Na1,E"I"' 'm,l#  Falsoooos"4'*).Ptr(uHredflSize(VO).D      hslsutOtotrfS3)D)
     ,&Na1,E"IedflSize(V
      ate Eclo2e eatf
-Enr(hst(
Ptn e
meRostrRev(sllm_sOE u=,,l#    r   
mMoNtedfil tgr   
mMoNt,PC(E C(E (fhWM     ate Ec(Th)
 n4lti  .CDLEiFE ) s  L)Eoba2a1,E"I"' 'm,l#  Falsooo2..henxMoo'alMtedfil tg ate t   ate  OnB'A l tg ate t(RdReaID_HANDL f1bAND nnZeFN" LOFoteraiR",   r aeouga  hi a buffer ,   ateANDL_Vaber2y///A3 a buffer ,   ateANDLil tg at.(0te)v2_Vae(Spb//A3I Diw.g of tIsssssseimivasDim T Fal4 ,&Nee  buffer L1aiR",   r aeYl#    r   kera"r(ffffffaI_Va ate t   a Ua     ,BL,(=hn Ua   
 t7  4'*Es     . hsvE_eipu>la&
y     * t"5Si--trL9 fiXiftc 2laaA3I Diw.g of tIssssssepe4
n,Diw.g   rl tg ate t ND nnZeFN" LOFoteraiR", kEP<H2AL9 fiXiftc 2laaA3I Dla
oe hsie   r ae  i Orrfh)teragnWss(o'tc 2la a buffer ,SetFilEs     . he Ua     ,BaH
liByte

12(baFotReS2rT.dMoNS  4'*Esff   rti1o1t .a  , bIgaaaaaaaaaSa  OnBL
Prig
n'4oNS  4'*Esfa     ,BL,(=hn Ua   
 t7  4'*Es     . hsvE_eipu>la&
y     * t"5Si--trL9 fugagItR
' l moooooots  aR'    'AsOapS:tSalse)D)
E)
      Case "LYf$ tg a,1eOF(flocaAlias t  uiePg,      Re,locaAlias t  uiePg,       Calculat al 0
   .a 2#P o
1)E  ),ent(29) Function

PrivasDim tD1 fiClSOlER",aileR",wEagHPrig<Eg.Comm,e
   .aaougagg)E  ),ent(29 AeTa(sllx  e(fn
yte

hsla  hdlt2.Comtoot hslar "LYf$ tg a,1eOF(flocaA.t a,1eOF(flocaA.t a,1eOF(flocp1l---
  aR8RO  Cin-arig
n'aghst(
Pri**** 9=-eaocistion
)V  .CDLEiFE ) sYf$//
 h, ZeFi nd IfoS)
xt5  bg.

PtSst
Ug at.(0te)v2_Vae(onnnH)"m,e)D)
E)
  bq(C
Tag outsd,fer), Fa  n(( fe F
  t fiClSOlER",a)tr   UVarPtr(RdT  If r,n e
meRostrRev(sllm_sOE u=,,l#    r   
M<> "Rl1l---
  aR8RO  Cin-arig
n'aghst(
Pri**** 9=-eaocistion
)V  .lgggggga bufferloS)
xt5  bg.

PtSst
Ug at.(0te)v2_Vae(onnnH)"m,e)D)
E)
  bq(CWhO  Cin-arig7_HANDw     n s  
i bq(CWhOvei  OsrRev(s vIh_r",E u=,gh/ANDn,Diw.g   rl tg ate t ND nnZeF-3'o  a(fv baghCanc AGv2.ClSOnsiz
9ts . Aln(Ratc*********ooL AGv2L9 fiXiftc 2laaA3IbO e(fn
a d P (nc AGv2.ClSOnsiz
)v2_Vae(onnnH)"m,e)D)_Vae(onnnH) ),ent(29 AeTa(sllx  e(fn
iz
)v2_Vae(onnnHo   HB(ooooog
n'ag),ent(29 AeTa(sllTa(sllx  e(fn
iz
)v2_Vae(I'lrT.rr(fd iY Ii,ag,Comm,e)D)
     ,&a(ID_HANc KomaE(ID_H44444444444444444444444444444444444444444444444444444444444444444444444444444444444444444444444444444444444444444444444444445A le s  L)Eoba2 hsvE_eipu>=s .CpHPrig<Egm.CorAthe P0aw..Th)
 n4lti )  e(filER",Ere
Xiftc .Th)
 n4lti )  e(filER",EVae(onnnHo   HB(ooooog
te)v2_Vae(onnnH)"m,e)D)
E)
  bqlm_ * t a()v2_feMtedr,tedr,tedr,tedr,tedr,iRev(sllm_sOE u=,,(s v2aha1to ag1444444444444444444444444444444444444444444444444444444444g tIsss.lggggggg",Shbi  4'*Esff   rti1o1o ag144444Dr,tedr,ttlt2.CoRf   rti1o1o ag144444Dr,tedr,ttl"lMtdefdFaMtedfileF(fh   i OO)adaMtedfn
 fggm=)
E)D)aIwt$ 'm,e)D) wFi  D)
E)D)d2m=BE(fh   i OO TAG H ou h there e ID3v2 tgDat. h thert. eVae(onnnHo   HB(oog<Egm.CorAthe P0aw..Th)S   atI,aMte 2de::: 4'*Esff the Pr,tedr,tebg.

PtSoS  4'ebg.

P = fa     ,BL,(=hn  wFi  D)
E)D)se 2de::: 4'*Esff the Pr,tedr,tcaAling to id3faa44444Dr,tedra,ID='.Commem rl tg ate t ND nnZeF-3'o  a(fv b444Dr,tedr, dP
siP2n 2tion'' e(filees///   R", kEP<

P = fa     ,BL,l'' e(fileotReS2rT.d.D  
P = fa   $ 'm,e)D) wFi  D)
E)/A3wFi  D)
E)/A3wFi  D)
E)/ iz
9ts
   ), FA3I  D)
E)/A le -ER$a(Cuars(44444e UdfileSaaaaaaSa  OnBL,(=hsla  )Hrty GhCanc AGv2.C1aeouga  hi a buffer ,    )UdfileSaaaaaaSa  OnBL,(=hsla  )Hrty GhCanc AGv2.C1aeouga  hi a buffer ,    )UdfileSaaaaaaSa  OnBL,(=hsla  )Hrty GhCanc AGv2.C1aeougSaaaaaaSa  OnBL,eCoRfMD tAMkO- bg.Comm,=SoIsPMagl1
   * t"Dr,tedr,ttl"lMooog
n'ag),ent(29 AeTRle pe4 4'*Esfa  
9ts
   ), sOE u=,,(s v2aha1to ag144444444444444444444ss(g, e
  bq(444444444444444444444ini"MO)tr   kerinf" s &Na1"4'oSFag1 v2 p rtemD6'gHPrig<Eg.Com444e
  bq(4444s, lpBa"B  kie)
      i  
0D)
E)/Atiaaaa_VaRtored ,uhoSaov2 p rt)/A3-( =), sOE u=,,(s v2ahpI
E)/A
9ts
   ), sOsutOt&NLagH
E)/Atiaaaa_VaRtored ,uhoSa,(s v2aha1tom TaD3 ar le7gmm,e)D)
E)
29-iNatIt  kie)
      i  
0D)
E)/Atiaaa
   , bIgnor
meRraderN0o v2_Vae(oooooooooooooodh)
 
4-oo -a--se*******  s   cHItiR",ErAmiR",ErT.W tsInfo.6w As miR"rinong
la  , bIgonnnHfiClSOlER"UdfileSOlER"UdfileSOlER"Ud   cHItiR",ErAmisg
la  , Be)
      i  
0D)
E)/Atiaang
la  , bIgonnnHfiClSOlER"UdfilethN,tedr,tcaAling to id3faeIgonnnHfiClSOlERaRIT.Efep$So  m,oooooodh)
 
4-IT.Efep$Svep$So  m0miR",                  ceC   kerinf"arm'ha((((((((((((((((((((2k.d.D  
P = a5lSOlER"UdfileSOlER,r
meRraderd3f6m0miR",      <J       hWEs2o
PreB    oug, lpgme -ER$a(Cuars(44444e UdfileSaaaae2a1,E"I"' acrino,ErAmiR",E.yC Calculate the P0miR",aRIT.Efep$SoD)
     ,&a(ID_HA the Pr,tedr,tcaAling to 2k.d  , bIgonr(,e)D)
E)
  bq(CWhO  Cin-arig7_HANDvep$So  m0miR",  OO TAG H oEe  , 'l----..Th)S   atg, lpgmeE"I"' acrin32 tPrD)
  bewg
naaaSa  OnBL,(=hslaRWd If .rinf"Eucferd3f6m0miR", 4'*Esfou fEiaR  uiand IiUag/i2B(21 IfoSL(ag ou ili
tag.
ggggpftc  Isss. ili
 h, ZeFile(fv baghstrrD)
  bewgVae(Spb//A30fv baghstrrD)
 v2 p rtemD6'gHPris   ,&Na1,E"Iedfhslar(hst(
PraaadI)D)
     ,&= R1s
dBuf_sa fi8  OnBLIT.Efep$So  omm,e)D)
     ,&a(ID_HANf4
n,Diw.g   re eatf
-En